// -*- C++ -*-
/*!
 * @file  PublisherFlush.h
 * @brief PublisherFlush class
 * @date  $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2010
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PublisherFlush.h 1971 2010-06-03 08:46:40Z n-ando $
 *
 */

#ifndef RTC_PUBLISHERFLUSH_H
#define RTC_PUBLISHERFLUSH_H

#include <coil/Condition.h>
#include <rtm/PublisherBase.h>
#include <rtm/SystemLogger.h>
#include <rtm/ConnectorBase.h>
#include <rtm/ConnectorListener.h>

namespace coil
{
  class Properties;
};

namespace RTC
{
  class InPortConsumer;

  /*!
   * @if jp
   * @class PublisherFlush
   * @brief PublisherFlush NX
   *
   * Flush ^ Publisher NX
   * obt@Ɋi[Ă関Mf[^𑗐MB
   * f[^o҂RV[}Ao鑤ƓXbhœ삳B
   *
   * @else
   * @class PublisherFlush
   * @brief PublisherFlush class
   *
   * This is a Publisher class of Flush type.
   * This class sends unsend data that has been stored in the buffer.
   * This executes Consumer that waits for the data send timing in the same 
   * thread as its send side.
   *
   * @endif
   */
  class PublisherFlush
    : public PublisherBase
  {
  public:
    typedef coil::Mutex Mutex;
    typedef coil::Condition<Mutex> Condition;
    typedef coil::Guard<coil::Mutex> Guard;
    DATAPORTSTATUS_ENUM

    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @else
     * @brief Constructor
     *
     * Consrtuctor.
     *
     * @endif
     */
    PublisherFlush();
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~PublisherFlush(void);

    /*!
     * @if jp
     * @brief 
     *
     * ̃NX̃IuWFNggp̂ɐ旧AK̊֐Ă
     * oKvBA PublisherFlush ͌ŏp
     * [^ȂB
     *    
     * @param property {Publisher̋쓮ݒ肵PropertyIuWFNg
     * @return ReturnCode PORT_OK I
     *                    INVALID_ARGS Properties sȒl܂
     *
     * @else
     * @brief initialization
     *
     * This function have to be called before using this class object.
     * However, this PublisherFlush class has no parameters to be initialized.
     *
     * @param property Property objects that includes the control information
     *                 of this Publisher
     * @return ReturnCode PORT_OK normal return
     *                    INVALID_ARGS Properties with invalid values.
     * @endif
     */
    virtual ReturnCode init(coil::Properties& prop);

    /*!
     * @if jp
     * @brief InPortRV[}̃Zbg
     *
     * ̊֐ł́A Publisher Ɋ֘AtRV[}ZbgB
     * RV[}IuWFNgk|C^̏ꍇAINVALID_ARGSԂB
     * ȊȌꍇ́APORT_OK ԂB
     *
     * @param consumer Consumer ւ̃|C^
     * @return ReturnCode PORT_OK I
     *                    INVALID_ARGS ɕsȒl܂܂Ă
     *
     * @else
     * @brief Store InPort consumer
     *
     * This operation sets a consumer that is associated with this
     * object. If the consumer object is NULL, INVALID_ARGS will be
     * returned.
     *
     * @param consumer A pointer to a consumer object.
     * @return ReturnCode PORT_OK normal return
     *                    INVALID_ARGS given argument has invalid value
     *
     * @endif
     */
    virtual ReturnCode setConsumer(InPortConsumer* consumer);

    /*!
     * @if jp
     * @brief obt@̃Zbg
     * 
     * PublisherFlushł́Aobt@gpȂ߁AȂꍇ
     * PORT_OK ԂB
     *
     * @param buffer CDRobt@
     * @return PORT_OK I
     *
     * @else
     * @brief Setting buffer pointer
     *
     * Since PublisherFlush does not use any buffers, This function
     * always returns PORT_OK.
     *
     * @param buffer CDR buffer
     * @return PORT_OK
     *
     * @endif
     */
    virtual ReturnCode setBuffer(CdrBufferBase* buffer);

    /*!
     * @if jp
     * @brief Xiݒ肷B
     *
     * Publisher ɑ΂ăXiIuWFNg ConnectorListeners ݒ肷B
     * e탊XiIuWFNg܂ ConnectorListeners Zbg邱ƂŁA
     * obt@̓ǂݏAf[^̑Mɂ̃XiR[B
     * ConnectorListeners IuWFNg̏L̓|[g܂ RTObject 
     * Publisher 폜 ConnectorListeners ͍폜邱Ƃ͂ȂB
     * ConnectorListeners k|C^̏ꍇ INVALID_ARGS ԂB
     *
     * @param info ConnectorProfile [JIuWFNg ConnectorInfo
     * @param listeners Xi𑽐ێ ConnectorListeners IuWFNg
     * @return PORT_OK      I
     *         INVALID_ARGS sȈ
     * @else
     * @brief Set the listener. 
     *
     * This function sets ConnectorListeners listener object to the
     * Publisher. By setting ConnectorListeners containing various
     * listeners objects, these listeners are called at the time of
     * reading and writing of a buffer, and transmission of data
     * etc. Since the ownership of the ConnectorListeners object is
     * owned by Port or RTObject, the Publisher never deletes the
     * ConnectorListeners object. If the given ConnectorListeners'
     * pointer is NULL, this function returns INVALID_ARGS.
     *
     * @param info ConnectorInfo that is localized object of ConnectorProfile
     * @param listeners ConnectorListeners that holds various listeners
     * @return PORT_OK      Normal return
     *         INVALID_ARGS Invalid arguments
     * @endif
     */
    virtual ::RTC::DataPortStatus::Enum
    setListener(ConnectorInfo& profile,
                RTC::ConnectorListeners* listeners);

    /*!
     * @if jp
     * @brief f[^
     *
     * Publisher ێRV[}ɑ΂ăf[^ށBR
     * V[}AXiK؂ɐݒ肳ĂȂAPublisher IuWF
     * NgĂȂꍇÅ֐ĂяoƃG[R[
     * h PRECONDITION_NOT_MET ԂARV[}ւ̏ݓ̑
     * ͈؍sȂB
     *
     * RV[}ւ݂̏ɑ΂āARV[}tԁAR
     * V[}̃G[ARV[}ւ݂̏^CAEgꍇ
     * ɂ͂ꂼAG[R[h SEND_FULL, SEND_ERROR, SEND_TIMEOUT
     * ԂB
     *
     * ȊÕG[̏ꍇAPORT_ERROR ԂB
     * 
     *
     * @param data ރf[^ 
     * @param sec ^CAEg
     * @param nsec ^CAEg
     *
     * @return PORT_OK             I
     *         PRECONDITION_NO_MET consumer, buffer, listenerK؂ɐݒ
     *                             ĂȂÃIuWFNg̎O
     *                             𖞂ȂꍇB
     *         SEND_FULL           M悪t
     *         SEND_TIMEOUT        M悪^CAEg
     *         CONNECTION_LOST     ڑؒfꂽƂmB
     *
     * @else
     * @brief Write data 
     *
     * This function writes data into the consumer associated with
     * this Publisher. If this function is called without initializing
     * correctly such as a consumer, listeners, etc., error code
     * PRECONDITION_NOT_MET will be returned and no operation of the
     * writing to the consumer etc. will be performed.
     *
     * When publisher writes data to the buffer, if the consumer
     * returns full-status, returns error, is returned with timeout,
     * error codes BUFFER_FULL, BUFFER_ERROR and BUFFER_TIMEOUT will
     * be returned respectively.
     *
     * In other cases, PROT_ERROR will be returned.
     *
     * @param data Data to be wrote to the buffer
     * @param sec Timeout time in unit seconds
     * @param nsec Timeout time in unit nano-seconds
     * @return PORT_OK             Normal return
     *         PRECONDITION_NO_MET Precondition does not met. A consumer,
     *                             a buffer, listenes are not set properly.
     *         SEND_FULL           Data was sent but full-status returned
     *         SEND_TIMEOUT        Data was sent but timeout occurred
     *         CONNECTION_LOST     detected that the connection has been lost
     *
     * @endif
     */
    virtual ReturnCode write(const cdrMemoryStream& data,
                             unsigned long sec,
                             unsigned long usec);
    /*!
     * @if jp
     *
     * @brief ANeBumF
     * 
     * Publisher ̓f[^|[gƓ activate/deactivate B
     * activate() / deactivate() ֐ɂāAANeBuԂƔANeB
     * uԂ؂ւB̊֐ɂA݃ANeBuԂAAN
     * eBuԂmF邱ƂłB
     *
     * @return ԊmF(ANeBu:trueAANeBu:false)
     *
     * @else
     *
     * @brief If publisher is active state
     * 
     * A Publisher can be activated/deactivated synchronized with the
     * data port.  The active state and the non-active state are made
     * transition by the "activate()" and the "deactivate()" functions
     * respectively. This function confirms if the publisher is in
     * active state.
     *
     * @return Result of state confirmation
     *         (Active state:true, Inactive state:false)
     *
     * @endif
     */
    virtual bool isActive();

    /*!
     * @if jp
     * @brief ANeBu
     *
     * Publisher ANeBuB̊֐ĂяoƂɂA
     * PublisherAf[^𑗐MXbhJnB
     * sĂȂȂǂɂAO𖞂ȂꍇAG[R[
     * h PRECONDITION_NOT_MET ԂB
     *
     * @return PORT_OK I
     *         PRECONDITION_NOT_MET O𖞂Ȃ
     *
     * @else
     * @brief activation
     *
     * This function activates the publisher. By calling this
     * function, this publisher starts the thread that pushes data to
     * InPort. If precondition such as initialization process and so
     * on is not met, the error code PRECONDITION_NOT_MET is returned.
     *
     * @return PORT_OK normal return
     *         PRECONDITION_NOT_MET precondition is not met
     *
     * @endif
     */
    virtual ReturnCode activate();

    /*!
     * @if jp
     * @brief ANeBu
     *
     * Publisher ANeBuB̊֐ĂяoƂɂA
     * PublisherAf[^𑗐MXbh~B
     * sĂȂȂǂɂAO𖞂ȂꍇAG[R[
     * h PRECONDITION_NOT_MET ԂB
     *
     * @return PORT_OK I
     *         PRECONDITION_NOT_MET O𖞂Ȃ
     *
     * @else
     * @brief deactivation
     *
     * This function deactivates the publisher. By calling this
     * function, this publisher stops the thread that pushes data to
     * InPort. If precondition such as initialization process and so
     * on is not met, the error code PRECONDITION_NOT_MET is returned.
     *
     * @return PORT_OK normal return
     *         PRECONDITION_NOT_MET precondition is not met
     *
     * @endif
     */
    virtual ReturnCode deactivate();

  protected:
    /*!
     * @if jp
     * @brief ON_SEND̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_SEND event to listners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onSend(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_SEND].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_RECEIVED̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_RECEIVED event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onReceived(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_RECEIVED].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_RECEIVER_FULL̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_RECEIVER_FULL event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onReceiverFull(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_RECEIVER_FULL].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_RECEIVER_TIMEOUT̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_RECEIVER_TIMEOUT event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onReceiverTimeout(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_RECEIVER_TIMEOUT].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_RECEIVER_ERROR̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_RECEIVER_ERROR event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onReceiverError(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_RECEIVER_ERROR].notify(m_profile, data);
    }
    
  private:
    Logger rtclog;
    InPortConsumer* m_consumer;
    ConnectorInfo m_profile;
    ConnectorListeners* m_listeners;
    ReturnCode m_retcode;
    Mutex m_retmutex;
    bool m_active;
  };

};     // namespace RTC

extern "C"
{
  void DLL_EXPORT PublisherFlushInit();
};

#endif // RTC_PUBLISHERFLUSH_H

