// -*- C++ -*-
/*!
 * @file RTObject.h
 * @brief RT component base class
 * @date $Date: 2008-01-14 07:57:18 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2010
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RTObject.h 2209 2011-07-27 14:02:57Z n-ando $
 *
 */

#ifndef RTC_RTOBJECT_H
#define RTC_RTOBJECT_H

// CORBA header include
#include <coil/Properties.h>

#include <rtm/RTC.h>
#include <rtm/idl/RTCSkel.h>
#include <rtm/idl/OpenRTMSkel.h>
#include <rtm/PortBase.h>
#include <rtm/PortAdmin.h>
#include <rtm/InPortBase.h>
#include <rtm/OutPortBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/ConfigAdmin.h>
#include <rtm/SystemLogger.h>
#include <rtm/ComponentActionListener.h>
#include <rtm/SdoServiceAdmin.h>
#include <rtm/PortConnectListener.h>

#define ECOTHER_OFFSET 1000

namespace SDOPackage
{
  class Configuration_impl;
};

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

namespace RTC
{
  class Manager;
  class ExecutionContextBase;
  typedef ExecutionContextHandle_t UniqueId;

  /*!
   * @if jp
   * @brief RTR|[lgNX
   *
   * eRTR|[lg̃x[XƂȂNXB
   * Robotic Technology Component dl lightweightRTComponent̎NXB
   * R|[lg̋@\񋟂 ComponentAction C^[tF[X
   * R|[lg̃CtTCNǗs߂ LightweightRTObject ̎
   * 񋟂B
   * ۂɃ[UR|[lg쐬ꍇɂ́AExecution Semantics ɑΉ
   * eTuNX𗘗pB<BR>
   * (̎ł Periodic Sampled Data Processing ̂݃T|[gĂ邽߁A
   *  dataFlowComponent 𒼐ڌpĂ)
   *
   * @since 0.2.0
   *
   * @else
   * @brief RT-Component class
   *
   * This is a class to be a base of each RT-Component.
   * This is a implementation class of lightweightRTComponent in Robotic
   * Technology Component specification.
   * This provides a implementation of ComponentAction interface that offers
   * the the component's function and the implementation of LightweightRTObject
   * for management of the component's lifecycle.
   * When users actually create the components, they should use each subclass
   * corresponding to Execution Semantics.<BR>
   * (In current implementation, since only Periodic Sampled Data Processing is
   * supported, this class inherits dataFlowComponent directly.)
   *
   * @since 0.2.0
   *
   * @endif
   */
  class RTObject_impl
    : public virtual POA_OpenRTM::DataFlowComponent, 
      public virtual PortableServer::RefCountServantBase
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param manager }l[WIuWFNg
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor
     *
     * @param manager Manager object
     *
     * @endif
     */
    RTObject_impl(Manager* manager);
    
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param orb ORB
     * @param poa POA
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor
     *
     * @param orb ORB
     * @param poa POA
     *
     * @endif
     */
    RTObject_impl(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa);
    
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * @else
     * 
     * @brief Virtual destructor
     * 
     * @endif
     */
    virtual ~RTObject_impl(void);
    
  protected:
    //============================================================
    // Overridden functions
    //============================================================
    /*!
     * @if jp
     *
     * @brief pR[obN֐
     * 
     * ComponentAction::on_initialize Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̏́A{֐I[o[ChĎ
     * KvB
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to initialize
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_initialize was invoked.<BR>
     * As for actual initialization of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The initialize action (on CREATED->ALIVE transition)
    // formaer rtc_init_entry() 
    virtual ReturnCode_t onInitialize();
    
    /*!
     * @if jp
     *
     * @brief IpR[obN֐
     * 
     * ComponentAction::on_finalize Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̏ÍA{֐I[o[ChĎ
     * KvB
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to finalize
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_finalize was invoked.<BR>
     * As for actual finalization of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The finalize action (on ALIVE->END transition)
    // formaer rtc_exiting_entry()
    virtual ReturnCode_t onFinalize();
    
    /*!
     * @if jp
     *
     * @brief JnpR[obN֐
     * 
     * ComponentAction::on_startup Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̊JńA{֐I[o[ChĎ
     * KvB
     * 
     * @param exec_handle QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function for startup action
     * 
     * Callback function that is executed when
     * ComponentAction::on_startup was invoked.<BR>
     * As for actual startup of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param exec_handle ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The startup action when ExecutionContext startup
    // former rtc_starting_entry()
    virtual ReturnCode_t onStartup(RTC::UniqueId exec_handle);
    
    /*!
     * @if jp
     *
     * @brief ~pR[obN֐
     * 
     * ComponentAction::on_shutdown Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̒~́A{֐I[o[ChĎ
     * KvB
     * 
     * @param exec_handle QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function for shutdown action
     * 
     * Callback function that is executed when
     * ComponentAction::on_shutdown was invoked.<BR>
     * As for actual shutdown of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param exec_handle ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The shutdown action when ExecutionContext stop
    // former rtc_stopping_entry()
    virtual ReturnCode_t onShutdown(RTC::UniqueId exec_handle);
    
    /*!
     * @if jp
     *
     * @brief pR[obN֐
     * 
     * ComponentAction::on_activated Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̊́A{֐I[o[ChĎ
     * KvB
     * 
     * @param exec_handle QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to activate
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_activated was invoked.<BR>
     * As for actual activation of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param exec_handle ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The activated action (Active state entry action)
    // former rtc_active_entry()
    virtual ReturnCode_t onActivated(RTC::UniqueId exec_handle);
    
    /*!
     * @if jp
     *
     * @brief 񊈐pR[obN֐
     * 
     * ComponentAction::on_deactivated Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă
     * ŁAeR|[lg̎ۂ̔񊈐́A{֐I[o[C
     * hĎKvB
     * 
     * @param exec_handle QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to deactivate
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_deactivated was invoked.<BR>
     * As for actual deactivation of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param exec_handle ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The deactivated action (Active state exit action)
    // former rtc_active_exit()
    virtual ReturnCode_t onDeactivated(RTC::UniqueId exec_handle);
    
    /*!
     * @if jp
     *
     * @brief pR[obN֐
     * 
     * DataFlowComponentAction::on_execute Ă΂ꂽۂɎs
     * R[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̎́A{֐I[o[ChĎ
     * KvB<BR>
     * {֐ Periodic Sampled Data Processing ɂ Two-Pass Execution
     * Pڂ̎spXƂĒIɌĂяoB
     * 
     * @param exec_handle QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to execute periodically
     * 
     * This is a callback function that is executed when
     * DataFlowComponentAction::on_execute is invoked.<BR>
     * As for actual periodic execution of each component, since this
     * function is dummy-implemented to return RTC::RTC_OK
     * unconditionally, you need to implement this function by
     * overriding it.  This function is invoked periodically as the
     * first execution pass of Two-Pass Execution in Periodic Sampled
     * Data Processing.
     * 
     * @param exec_handle ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The execution action that is invoked periodically
    // former rtc_active_do()
    virtual ReturnCode_t onExecute(RTC::UniqueId exec_handle);
    
    /*!
     * @if jp
     *
     * @brief fpR[obN֐
     * 
     * ComponentAction::on_aborting Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̒f́A{֐I[o[ChĎ
     * KvB
     * 
     * @param exec_handle QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to abort
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_aborting was invoked.<BR>
     * As for actual abortion of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param exec_handle ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The aborting action when main logic error occurred.
    // former rtc_aborting_entry()
    virtual ReturnCode_t onAborting(RTC::UniqueId exec_handle);
    
    /*!
     * @if jp
     *
     * @brief G[pR[obN֐
     * 
     * ComponentAction::on_error Ă΂ꂽۂɎsR[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̃G[́A{֐I[o[ChĎ
     * KvB
     * 
     * @param exec_handle QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function for error handling
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_error was invoked.<BR>
     * As for actual error handling of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param exec_handle ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The error action in ERROR state
    // former rtc_error_do()
    virtual ReturnCode_t onError(RTC::UniqueId exec_handle);
    
    /*!
     * @if jp
     *
     * @brief ZbgpR[obN֐
     * 
     * ComponentAction::on_reset Ă΂ꂽۂɎsR[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̃ZbǵA{֐I[o[Ch
     * ĎKvB
     * 
     * @param exec_handle QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to reset
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_reset was invoked.<BR>
     * As for actual reset of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param exec_handle ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The reset action that is invoked resetting
    // This is same but different the former rtc_init_entry()
    virtual ReturnCode_t onReset(RTC::UniqueId exec_handle);
    
    /*!
     * @if jp
     *
     * @brief ԕύXpR[obN֐
     * 
     * DataFlowComponentAction::on_state_update Ă΂ꂽۂɎs
     * R[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̏ԕύX́A{֐I[o[Ch
     * ĎKvB<BR> {֐ Periodic Sampled Data
     * Processing ɂ Two-Pass Execution̂Qڂ̎spXƂĒ
     * IɌĂяoB
     *
     * @param exec_handle QĂ ExecutionContext  ID
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to update the state
     * 
     * This is a callback function that is executed when
     * DataFlowComponentAction::on_state_update was invoked.<BR>
     * As for actual updating the state of each component, since this
     * function is dummy-implemented to return RTC::RTC_OK
     * unconditionally, you need to implement this function by
     * overriding it.<BR> This function is invoked periodically as the
     * second execution pass of Two-Pass Execution in Periodic Sampled
     * Data Processing.
     *
     * @param exec_handle ID of the participant ExecutionContext
     * 
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The state update action that is invoked after onExecute() action
    // no corresponding operation exists in OpenRTm-aist-0.2.0
    virtual ReturnCode_t onStateUpdate(RTC::UniqueId exec_handle);
    
    /*!
     * @if jp
     *
     * @brief ύXʒmpR[obN֐
     * 
     * DataFlowComponentAction::on_rate_changed Ă΂ꂽۂɎs
     * R[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̏ԕύX́A{֐I[o[Ch
     * ĎKvB<BR> {֐ Periodic Sampled Data
     * Processing ɂ ExecutionContext ̎sXVꂽۂɌĂ
     * oB
     *
     * @param exec_handle QĂ ExecutionContext  ID
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to change execution cycle
     * 
     * This is a callback function that is executed when
     * DataFlowComponentAction::on_rate_changed was invoked.<BR>
     * As for actual changing execution cycle of each component, since this 
     * function is dummy-implemented to return RTC::RTC_OK unconditionally,
     * you need to implement this function by overriding it.<BR>
     * This function is invoked when the execution of ExecutionContext 
     * was updated in Periodic Sampled Data Processing.
     *
     * @param exec_handle ID of the participant ExecutionContext
     * 
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The action that is invoked when execution context's rate is changed
    // no corresponding operation exists in OpenRTm-aist-0.2.0
    virtual ReturnCode_t onRateChanged(RTC::UniqueId exec_handle);
    
  public:
    //============================================================
    // RTC::LightweightRTObject
    //============================================================
    /*!
     * @if jp
     *
     * @brief [CORBA interface] RTC
     *
     * ̃Iy[VĂяǒʂƂāAComponentAction::on_initialize
     * R[obN֐Ă΂B
     * 
     * 
     * - RTC  CreatedԂ̏ꍇݏsB̏Ԃɂꍇɂ
     *   ReturnCode_t::PRECONDITION_NOT_MET ԂĂяo͎sB
     * - ̃Iy[V RTC ̃~hEGAĂ΂邱Ƃz肵ĂA
     *   AvP[VJ҂͒ڂ̃Iy[VĂԂƂ͑z
     *   ĂȂB
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief [CORBA interface] Initialize the RTC that realizes this interface.
     *
     * The invocation of this operation shall result in the invocation of the
     * callback ComponentAction::on_initialize.
     *
     * Constraints
     * - An RTC may be initialized only while it is in the Created state. Any
     *   attempt to invoke this operation while in another state shall fail
     *   with ReturnCode_t::PRECONDITION_NOT_MET.
     * - Application developers are not expected to call this operation
     *   directly; it exists for use by the RTC infrastructure.
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    virtual ReturnCode_t initialize()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] RTC I
     *
     * ̃Iy[VĂяǒʂƂ ComponentAction::on_finalize()
     * ĂяoB
     *
     * 
     * - RTC  ExecutionContext ɏĂԂ͏IȂB̏ꍇ́A
     *   ܂ŏ ExecutionContextOperations::remove ɂĎQ
     *   Ȃ΂ȂȂBȊȌꍇ́ÃIy[VĂяo
     *   Ȃꍇ ReturnCode_t::PRECONDITION_NOT_ME ŎsB
     * - RTC  Created ԂłꍇAI͍sȂB
     *   ̏ꍇÃIy[VĂяo͂Ȃꍇ
     *   ReturnCode_t::PRECONDITION_NOT_MET ŎsB
     * - ̃Iy[VRTC̃~hEGAĂ΂邱Ƃz肵ĂA
     *   AvP[VJ҂͒ڂ̃Iy[VĂԂƂ͑z
     *   ĂȂB
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief [CORBA interface] Finalize the RTC for destruction
     * 
     * This invocation of this operation shall result in the invocation of the
     * callback ComponentAction::on_finalize.
     *
     * Constraints
     * - An RTC may not be finalized while it is participating in any execution
     *   context. It must first be removed with 
     *   ExecutionContextOperations::remove. Otherwise, this operation
     *   shall fail with ReturnCode_t::PRECONDITION_NOT_MET. 
     * - An RTC may not be finalized while it is in the Created state. Any 
     *   attempt to invoke this operation while in that state shall fail with 
     *   ReturnCode_t::PRECONDITION_NOT_MET.
     * - Application developers are not expected to call this
     *   operation directly; it exists for use by the RTC
     *   infrastructure.
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    virtual ReturnCode_t finalize()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] RTC I[i[ł ExecutionContext 
     *        ~ÃRecƋɏI
     *
     *  RTC I[i[ł邷ׂĂ̎sReLXg~B
     *  RTC ̎sReLXgL RTC ɑsReLXg
     * (i.e. sReLXgL RTC ͂Ȃ킿̎sReLXg
     * I[i[łB)ɎQĂꍇAY RTC ͂̃ReLXg
     * Ŕ񊈐Ȃ΂ȂȂB
     * RTC ŝǂ ExecutionContext ł Active Ԃł͂ȂȂA
     *  RTC ƂɊ܂܂ RTC IB
     * 
     * 
     * - RTC ĂȂ΁AI邱Ƃ͂łȂB
     *   Created Ԃɂ RTC  exit() ĂяoꍇA
     *   ReturnCode_t::PRECONDITION_NOT_MET ŎsB
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief [CORBA interface]top the RTC's execution context(s) and finalize
     *        it along with its contents.
     * 
     * Any execution contexts for which the RTC is the owner shall be stopped. 
     * If the RTC participates in any execution contexts belonging to another
     * RTC that contains it, directly or indirectly (i.e. the containing RTC
     * is the owner of the ExecutionContext), it shall be deactivated in those
     * contexts.
     * After the RTC is no longer Active in any Running execution context, it
     * and any RTCs contained transitively within it shall be finalized.
     *
     * Constraints
     * - An RTC cannot be exited if it has not yet been initialized. Any
     *   attempt to exit an RTC that is in the Created state shall fail with
     *   ReturnCode_t::PRECONDITION_NOT_MET.
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    virtual ReturnCode_t exit()
      throw (CORBA::SystemException); 
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] RTC  Alive Ԃł邩ǂmFB
     *
     * RTC w肵 ExecutionContext ɑ΂ AliveԂł邩ǂ
     * mFBRTC ̏Ԃ Active ł邩AInactive ł邩A
     * Error ł邩͎sExecutionContext ɈˑBȂ킿A
     *  ExecutionContext ɑ΂ĂActive ԂłĂA
     * ExecutionContext ɑ΂Ă Inactive ԂƂȂꍇ肦B
     * ]āÃIy[V͎w肳ꂽExecutionContext ɖ₢
     * 킹āA RTC ̏Ԃ ActiveAInactiveAError ̏ꍇɂ
     * Alive ԂƂĕԂB
     *
     * @return Alive ԊmF
     *
     * @else
     *
     * @brief [CORBA interface] Confirm whether RTC is the alive state
     *
     * A component is alive or not regardless of the execution context from
     * which it is observed. However, whether or not it is Active, Inactive,
     * or in Error is dependent on the execution context(s) in which it is
     * running. That is, it may be Active in one context but Inactive in
     * another. Therefore, this operation shall report whether this RTC is
     * either Active, Inactive or in Error; which of those states a component
     * is in with respect to a particular context may be queried from the
     * context itself.
     *
     * @return Result of Alive state confirmation
     *
     * @endif
     */
    virtual CORBA::Boolean is_alive(ExecutionContext_ptr exec_context)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     * @brief [CORBA interface] ExecutionContext擾
     *
     * w肵nh ExecutionContext 擾B
     * nh ExecutionContext ւ̃}bsÓA RTC CX^X
     * ŗLłBnh͂ RTC  attach_context ۂɎ擾łB
     *
     * @param exec_handle 擾Ώ ExecutionContext nh
     *
     * @return ExecutionContext
     *
     * @else
     * @brief [CORBA interface] Get ExecutionContext.
     *
     * Obtain a reference to the execution context represented by the given 
     * handle.
     * The mapping from handle to context is specific to a particular RTC 
     * instance. The given handle must have been obtained by a previous call to 
     * attach_context on this RTC.
     *
     * @param exec_handle ExecutionContext handle
     *
     * @return ExecutionContext
     *
     * @endif
     */
    virtual ExecutionContext_ptr get_context(UniqueId exec_handle)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief [CORBA interface] L ExecutionContextList 擾
     *
     *  RTC L ExecutionContext ̃Xg擾B
     *
     * @return ExecutionContext Xg
     *
     * @else
     * @brief [CORBA interface] Get ExecutionContextList.
     *
     * This operation returns a list of all execution contexts owned by this
     * RTC.
     *
     * @return ExecutionContext List
     *
     * @endif
     */
    virtual ExecutionContextList* get_owned_contexts()
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief [CORBA interface] QĂ ExecutionContextList 擾
     *
     *  RTC QĂ ExecutionContext ̃Xg擾B
     *
     * @return ExecutionContext Xg
     *
     * @else
     * @brief [CORBA interface] Get participating ExecutionContextList.
     *
     * This operation returns a list of all execution contexts in
     * which this RTC participates.
     *
     * @return ExecutionContext List
     *
     * @endif
     */
    virtual ExecutionContextList* get_participating_contexts()
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief [CORBA interface] ExecutionContext ̃nhԂ
     *
     * ^ꂽsReLXgɊ֘AtꂽnhԂB
     *
     * @else
     * @brief [CORBA interface] Return a handle of a ExecutionContext
     *
     * This operation returns a handle that is associated with the given
     * execution context.
     *
     * @endif
     */
    virtual ExecutionContextHandle_t
    get_context_handle(ExecutionContext_ptr cxt)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief [CORBA interface] ExecutionContextattach
     *
     * w肵 ExecutionContext ɂ RTC B RTC Ɗ֘A 
     * ExecutionContext ̃nhԂB
     * ̃Iy[V́AExecutionContextOperations::add_component
     * Ă΂ꂽۂɌĂяoBԂꂽnh͑̃NCAg
     * gp邱Ƃz肵ĂȂB
     *
     * @param exec_context  ExecutionContext
     *
     * @return ExecutionContext nh
     *
     * @else
     * @brief [CORBA interface] Attach ExecutionContext
     *
     * Inform this RTC that it is participating in the given execution context. 
     * Return a handle that represents the association of this RTC with the 
     * context.
     * This operation is intended to be invoked by 
     * ExecutionContextOperations::add_component. It is not intended for use by 
     * other clients.
     *
     * @param exec_context Participating ExecutionContext
     *
     * @return ExecutionContext Handle
     *
     * @endif
     */
    UniqueId attach_context(ExecutionContext_ptr exec_context)
      throw (CORBA::SystemException);

    UniqueId bindContext(ExecutionContext_ptr exec_context);
    
    /*!
     * @if jp
     * @brief [CORBA interface] ExecutionContextdetach
     *
     * w肵 ExecutionContext 炱 RTC ̏B
     * ̃Iy[V́AExecutionContextOperations::remove Ă
     * ꂽۂɌĂяoBԂꂽnh͑̃NCAgŎgp邱Ƃ
     * z肵ĂȂB
     * 
     * 
     * - w肳ꂽ ExecutionContext  RTC łɏĂȂꍇɂ́A
     *   ReturnCode_t::PRECONDITION_NOT_MET ԂB
     * - w肳ꂽ ExecutionContext ɂđ΂ RTC Active Ԃł
     *   ɂ́A ReturnCode_t::PRECONDITION_NOT_MET ԂB
     *
     * @param exec_handle Ώ ExecutionContextnh
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     * @brief [CORBA interface] Detach ExecutionContext.
     *
     * Inform this RTC that it is no longer participating in the given
     * execution context.
     * This operation is intended to be invoked by 
     * ExecutionContextOperations::remove. It is not intended for use 
     * by other clients.
     * Constraints
     * - This operation may not be invoked if this RTC is not already 
     *   participating in the execution context. Such a call shall fail with 
     *   ReturnCode_t::PRECONDITION_NOT_MET.
     * - This operation may not be invoked if this RTC is Active in
     *   the indicated execution context. Otherwise, it shall fail
     *   with ReturnCode_t::PRECONDITION_NOT_MET.
     *
     * @param exec_handle Detaching ExecutionContext Handle
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    ReturnCode_t detach_context(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    //============================================================
    // RTC::RTObject
    //============================================================
    /*!
     * @if jp
     *
     * @brief [RTObject CORBA interface] R|[lgvt@C擾
     *
     * YR|[lg̃vt@CԂB 
     *
     * @return R|[lgvt@C
     *
     * @else
     *
     * @brief [RTObject CORBA interface] Get RTC's profile
     *
     * This operation returns the ComponentProfile of the RTC.
     *
     * @return ComponentProfile
     *
     * @endif
     */
    virtual ComponentProfile* get_component_profile()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [RTObject CORBA interface] |[g擾
     *
     * YR|[lgۗL|[g̎QƂԂB
     *
     * @return |[gXg
     *
     * @else
     *
     * @brief [RTObject CORBA interface] Get Ports
     *
     * This operation returns the reference of ports held by RTC.
     *
     * @return PortServiceList
     *
     * @endif
     */
    virtual PortServiceList* get_ports()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [RTObject CORBA interface] ExecutionContextAdmin 擾
     *
     * ̃Iy[V͓Y RTC  ExecutionContextɊ֘A
     * ExecutionContextService ̃XgԂB
     *
     * @return ExecutionContextService Xg
     *
     * @else
     *
     * @brief [RTObject CORBA interface] Get ExecutionContextAdmin
     *
     * This operation returns a list containing an ExecutionContextAdmin for
     * every ExecutionContext owned by the RTC.	
     *
     * @return ExecutionContextService List
     *
     * @endif
     */
    //    virtual ExecutionContextServiceList* get_execution_context_services()
    //      throw (CORBA::SystemException);
    
    //============================================================
    // RTC::ComponentAction
    //============================================================
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̏
     *
     * RTC AAlive ԂɑJڂB
     * RTC ŗL̏͂ŎsB
     * ̃Iy[VĂяǒʂƂ onInitialize() R[obN֐
     * ĂяoB
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Initialize RTC
     *
     * The RTC has been initialized and entered the Alive state.
     * Any RTC-specific initialization logic should be performed here.
     * As a result of this operation, onInitialize() callback function
     * is called.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_initialize()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̏I
     *
     * RTC jB
     * RTC ŗL̏I͂ŎsB
     * ̃Iy[VĂяǒʂƂ onFinalize() R[obN֐
     * ĂяoB
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Finalize RTC
     *
     * The RTC is being destroyed.
     * Any final RTC-specific tear-down logic should be performed here.
     * As a result of this operation, onFinalize() callback function is called.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_finalize()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̊Jn
     *
     * RTC  ExecutionContext  Stopped Ԃ Running Ԃ֑J
     * ꍇɌĂяoB
     * ̃Iy[VĂяǒʂƂ onStartup() R[obN֐
     * ĂяoB
     *
     * @param exec_handle ԑJڂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Startup RTC
     *
     * The given execution context, in which the RTC is participating, has 
     * transitioned from Stopped to Running.
     * As a result of this operation, onStartup() callback function is called.
     *
     * @param exec_handle ID of ExecutionContext that transited to the state
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_startup(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̒~
     *
     * RTC  ExecutionContext  Running Ԃ Stopped Ԃ֑J
     * ꍇɌĂяoB
     * ̃Iy[VĂяǒʂƂ onShutdown() R[obN֐
     * ĂяoB
     *
     * @param exec_handle ԑJڂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Shutdown RTC
     *
     * The given execution context, in which the RTC is participating, has 
     * transitioned from Running to Stopped.
     * As a result of this operation, onShutdown() callback function is called.
     *
     * @param exec_handle ID of ExecutionContext that transited to the state
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_shutdown(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̊
     *
     *  ExecutionContext  RTC ꂽۂɌĂяoB
     * ̃Iy[VĂяǒʂƂ onActivated() R[obN֐
     * ĂяoB
     *
     * @param exec_handle  ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Activate RTC
     *
     * The RTC has been activated in the given execution context.
     * As a result of this operation, onActivated() callback function is called.
     *
     * @param exec_handle ID of activation ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_activated(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̔񊈐
     *
     *  ExecutionContext  RTC 񊈐ꂽۂɌĂяoB
     * ̃Iy[VĂяǒʂƂ onDeactivated() R[obN֐
     * ĂяoB
     *
     * @param exec_handle 񊈐 ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Deactivate RTC
     *
     * The RTC has been deactivated in the given execution context.
     * As a result of this operation, onDeactivated() callback function
     * is called.
     *
     * @param exec_handle ID of deactivation ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_deactivated(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̃G[Ԃւ̑J
     *
     * RTC  ExecutionContext  Active Ԃ Error Ԃ֑Jڂ
     * ꍇɌĂяoB
     * ̃Iy[V RTC  Error ԂɑJڂۂɈxĂ
     * oB̃Iy[VĂяǒʂƂ onAborting() R[
     * obN֐ĂяoB
     *
     * @param exec_handle ԑJڂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Transition to Error State
     *
     * The RTC is transitioning from the Active state to the Error state in some
     * execution context.
     * This callback is invoked only a single time for time that the RTC 
     * transitions into the Error state from another state. This behavior is in 
     * contrast to that of on_error.
     * As a result of this operation, onAborting() callback function is invoked.
     *
     * @param exec_handle ID of ExecutionContext that transited to the state
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_aborting(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̃G[
     *
     * RTC G[ԂɂۂɌĂяoB
     * RTC G[Ԃ̏ꍇɁAΏۂƂȂ ExecutionContext ExecutionKind 
     * ^C~OŌĂяoBႦ΁A
     * - ExecutionKind  PERIODIC ̏ꍇA{Iy[V
     *   DataFlowComponentAction::on_execute  on_state_update ̑ւɁA
     *   ݒ肳ꂽԁAݒ肳ꂽŌĂяoB
     * - ExecutionKind  EVENT_DRIVEN ̏ꍇA{Iy[V
     *   FsmParticipantAction::on_action Ă΂ꂽۂɁAւɌĂяoB
     * ̃Iy[VĂяǒʂƂ onError() R[obN֐Ăяo
     * B
     *
     * @param exec_handle Ώ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Error Processing of RTC
     *
     * The RTC remains in the Error state.
     * If the RTC is in the Error state relative to some execution context when
     * it would otherwise be invoked from that context (according to the 
     * contextfs ExecutionKind), this callback shall be invoked instead. 
     * For example,
     * - If the ExecutionKind is PERIODIC, this operation shall be invoked in 
     *   sorted order at the rate of the context instead of 
     *   DataFlowComponentAction::on_execute and on_state_update.
     * - If the ExecutionKind is EVENT_DRIVEN, this operation shall be invoked 
     *   whenever FsmParticipantAction::on_action would otherwise have been 
     *   invoked.
     * As a result of this operation, onError() callback function is invoked.
     *
     * @param exec_handle ID of target ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_error(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̃Zbg
     *
     * Error Ԃɂ RTC ̃JosAInactive ԂɕA
     * ꍇɌĂяoB
     * RTC ̃Joꍇ Inactive ԂɕA邪AȊO
     * ꍇɂ Error Ԃɗ܂B
     * ̃Iy[VĂяǒʂƂ onReset() R[obN֐Ă
     * oB
     *
     * @param exec_handle ZbgΏ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Resetting RTC
     *
     * The RTC is in the Error state. An attempt is being made to
     * recover it such that it can return to the Inactive state.
     * If the RTC was successfully recovered and can safely return to
     * the Inactive state, this method shall complete with
     * ReturnCode_t::OK. Any other result shall indicate that the RTC
     * should remain in the Error state.  As a result of this
     * operation, onReset() callback function is invoked.
     *
     * @param exec_handle ID of target ExecutionContext for the reset
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_reset(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    //============================================================
    // RTC::DataFlowComponentAction
    //============================================================
    /*!
     * @if jp
     *
     * @brief [DataFlowComponentAction CORBA interface] RTC ̒폈()
     *
     * ȉ̏ԂێĂꍇɁAݒ肳ꂽŒIɌĂяoB
     * - RTC  Alive ԂłB
     * - w肳ꂽ ExecutionContext  Running ԂłB
     * {Iy[V́ATwo-Pass Execution ̑ŎsB
     * ̃Iy[VĂяǒʂƂ onExecute() R[obN֐Ă
     * oB
     *
     * 
     * - w肳ꂽ ExecutionContext  ExecutionKind ́A PERIODIC łȂ΂
     *   Ȃ
     *
     * @param exec_handle 폈Ώ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [DataFlowComponentAction CORBA interface] Primary Periodic 
     *        Operation of RTC
     *
     * This operation will be invoked periodically at the rate of the given
     * execution context as long as the following conditions hold:
     * - The RTC is Active.
     * - The given execution context is Running
     * This callback occurs during the first execution pass.
     * As a result of this operation, onExecute() callback function is invoked.
     *
     * Constraints
     * - The execution context of the given context shall be PERIODIC.
     *
     * @param exec_handle ID of target ExecutionContext for Primary
     *                    Periodic Operation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_execute(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [DataFlowComponentAction CORBA interface] RTC ̒폈()
     *
     * ȉ̏ԂێĂꍇɁAݒ肳ꂽŒIɌĂяoB
     * - RTC  Alive ԂłB
     * - w肳ꂽ ExecutionContext  Running ԂłB
     * {Iy[V́ATwo-Pass Execution ̑ŎsB
     * ̃Iy[VĂяǒʂƂ onStateUpdate() R[obN֐
     * ĂяoB
     *
     * 
     * - w肳ꂽ ExecutionContext  ExecutionKind ́A PERIODIC łȂ΂
     *   Ȃ
     *
     * @param exec_handle 폈Ώ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [DataFlowComponentAction CORBA interface] Secondary Periodic 
     *        Operation of RTC
     *
     * This operation will be invoked periodically at the rate of the given
     * execution context as long as the following conditions hold:
     * - The RTC is Active.
     * - The given execution context is Running
     * This callback occurs during the second execution pass.
     * As a result of this operation, onStateUpdate() callback function is
     * invoked.
     *
     * Constraints
     * - The execution context of the given context shall be PERIODIC.
     *
     * @param exec_handle ID of target ExecutionContext for 
     *              Secondary Periodic Operation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_state_update(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [DataFlowComponentAction CORBA interface] sύXʒm
     *
     * {Iy[V́AExecutionContext ̎sύXꂽƂʒm
     * ۂɌĂяoB
     * ̃Iy[VĂяǒʂƂ onRateChanged() R[obN֐
     * ĂяoB
     *
     * 
     * - w肳ꂽ ExecutionContext  ExecutionKind ́A PERIODIC łȂ΂
     *   Ȃ
     *
     * @param exec_handle 폈Ώ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [DataFlowComponentAction CORBA interface] Notify rate changed
     *
     * This operation is a notification that the rate of the indicated execution 
     * context has changed.
     * As a result of this operation, onRateChanged() callback function is 
     * called.
     *
     * Constraints
     * - The execution context of the given context shall be PERIODIC.
     *
     * @param exec_handle ID of target ExecutionContext for Operation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_rate_changed(UniqueId exec_handle)
      throw (CORBA::SystemException);
    
    //============================================================
    // SDOPackage::SdoSystemElement
    //============================================================
    /*!
     * @if jp
     * 
     * @brief [SDO interface] Organization Xg̎擾 
     *
     * SDOSystemElement 0͂ȏ Organization L邱Ƃ
     * oB SDOSystemElement 1ȏ Organization LĂꍇ
     * ɂ́ÃIy[V͏L Organization ̃XgԂB
     * OrganizationLĂȂ΋̃XgԂB
     *
     * @return LĂ Organization Xg
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get Organization list
     *
     * SDOSystemElement can be the owner of zero or more organizations.
     * If the SDOSystemElement owns one or more Organizations, this operation
     * returns the list of Organizations that the SDOSystemElement owns.
     * If it does not own any Organization, it returns empty list.
     *
     * @return Owned Organization List
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::OrganizationList* get_owned_organizations()
      throw (CORBA::SystemException,
             SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    //============================================================
    // SDOPackage::SDO
    //============================================================
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO ID ̎擾
     *
     * SDO ID ԂIy[VB
     * ̃Iy[V͈ȉ̌^̗O𔭐B
     * 
     * @return    \[Xf[^fŒ`Ă SDO  ID
     * 
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get the SDO ID
     *
     * This operation returns id of the SDO.
     * This operation throws SDOException with one of the following types.
     *
     * @return    id of the SDO defined in the resource data model.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual char* get_sdo_id()
      throw (CORBA::SystemException,
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO ^Cv̎擾
     * 
     * SDO Type ԂIy[VB
     * ̃Iy[V͈ȉ̌^̗O𔭐B
     *
     * @return    \[Xf[^fŒ`Ă SDO  Type
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get SDO type
     *
     * This operation returns sdoType of the SDO.
     * This operation throws SDOException with one of the following types.
     *
     * @return    Type of the SDO defined in the resource data model.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual char* get_sdo_type()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO DeviceProfile Xg̎擾 
     *
     * SDO  DeviceProfile ԂIy[VB SDO n[hEGAfoCX
     * Ɋ֘AtĂȂꍇɂ́A DeviceProfile ԂB
     * ̃Iy[V͈ȉ̌^̗O𔭐B
     *
     * @return    SDO DeviceProfile
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get SDO DeviceProfile list
     *
     * This operation returns the DeviceProfile of the SDO. If the SDO does not
     * represent any hardware device, then a DeviceProfile with empty values
     * are returned.
     * This operation throws SDOException with one of the following types.
     *
     * @return    The DeviceProfile of the SDO.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::DeviceProfile* get_device_profile()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO ServiceProfile ̎擾 
     *
     * SDO LĂ Service  ServiceProfile ԂIy[VB
     * SDO T[rXLĂȂꍇɂ́ÃXgԂB
     * ̃Iy[V͈ȉ̌^̗O𔭐B
     * 
     * @return    SDO 񋟂SĂ Service  ServiceProfileB
     * 
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get SDO ServiceProfile
     * 
     * This operation returns a list of ServiceProfiles that the SDO has.
     * If the SDO does not provide any service, then an empty list is returned.
     * This operation throws SDOException with one of the following types.
     * 
     * @return    List of ServiceProfiles of all the services the SDO is
     *            providing.
     * 
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::ServiceProfileList* get_service_profiles()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] ServiceProfile̎擾 
     *
     *  "id" Ŏw肳ꂽÕT[rX ServiceProfile ԂB
     * 
     * @param     id SDO Service  ServiceProfile Ɋ֘AtꂽʎqB
     * 
     * @return    w肳ꂽ SDO Service  ServiceProfileB
     * 
     * @exception InvalidParameter "id" Ŏw肵 ServiceProfile ݂ȂB
     *                             "id"  nullB
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get specified ServiceProfile
     *
     * This operation returns the ServiceProfile that is specified by the
     * argument "id."
     * 
     * @param     id The identifier referring to one of the ServiceProfiles.
     * 
     * @return    The profile of the specified service.
     * 
     * @exception InvalidParameter The ServiceProfile that is specified by 
     *                             the argument 'id' does not exist or if 'id'
     *                             is 'null.'
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::ServiceProfile* get_service_profile(const char* id)
      throw (CORBA::SystemException, 
	     SDOPackage::InvalidParameter, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] w肳ꂽ SDO Service ̎擾
     *
     * ̃Iy[V͈ "id" Ŏw肳ꂽOɂċʂ
     * SDO  Service ւ̃IuWFNgQƂԂB SDO ɂ񋟂
     * Service ͂ꂼӂ̎ʎqɂʂB
     *
     * @param id SDO Service Ɋ֘AtꂽʎqB
     *
     * @return vꂽ SDO Service ւ̎QƁB
     *
     * 
     * @exception InvalidParameter "id" Ŏw肵 ServiceProfile ݂ȂB
     *                             "id"  nullB
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get specified SDO Service's reference
     *
     * This operation returns an object implementing an SDO's service that
     * is identified by the identifier specified as an argument. Different
     * services provided by an SDO are distinguished with different
     * identifiers. See OMG SDO specification Section 2.2.8, "ServiceProfile,"
     * on page 2-12 for more details.
     *
     * @param id The identifier referring to one of the SDO Service
     *
     * @return The reference requested to SDO Service.
     *
     * @exception InvalidParameter Argument gidh is null, or if the 
     *                             ServiceProfile that is specified by argument
     *                            gidh does not exist.
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::SDOService_ptr get_sdo_service(const char* id)
      throw (CORBA::SystemException, 
	     SDOPackage::InvalidParameter, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] Configuration IuWFNg̎擾 
     *
     * ̃Iy[V Configuration interface ւ̎QƂԂB
     * Configuration interface ͊e SDO Ǘ邽߂̃C^[tF[X
     * ЂƂłB̃C^[tF[X DeviceProfile, ServiceProfile,
     * Organization Œ`ꂽ SDO ̑lݒ肷邽߂ɎgpB
     * Configuration C^[tF[X̏ڍׂɂẮAOMG SDO specification
     *  2.3.5, p.2-24 QƂ̂ƁB
     *
     * @return SDO  Configuration C^[tF[Xւ̎Q
     *
     * @exception InterfaceNotImplemented SDOConfigurationC^[tF[X
     *                                    ȂB
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get Configuration object
     *
     * This operation returns an object implementing the Configuration
     * interface. The Configuration interface is one of the interfaces that
     * each SDO maintains. The interface is used to configure the attributes
     * defined in DeviceProfile, ServiceProfile, and Organization.
     * See OMG SDO specification Section 2.3.5, "Configuration Interface,"
     * on page 2-24 for more details about the Configuration interface.
     *
     * @return The Configuration interface of an SDO.
     *
     * @exception InterfaceNotImplemented The target SDO has no Configuration
     *                                    interface.
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual SDOPackage::Configuration_ptr get_configuration()
      throw (CORBA::SystemException, 
	     SDOPackage::InterfaceNotImplemented, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] Monitoring IuWFNg̎擾 
     *
     * ̃Iy[V Monitoring interface ւ̎QƂԂB
     * Monitoring interface  SDO ǗC^[tF[ẌłB
     * ̃C^[tF[X SDO ̃vpeBj^O邽߂
     * gpB
     * Monitoring interface ̏ڍׂɂĂ OMG SDO specification 
     * 2.3.7 "Monitoring Interface" p.2-35 QƂ̂ƁB
     *
     * @return SDO  Monitoring interface ւ̎Q
     *
     * @exception InterfaceNotImplemented SDOConfigurationC^[tF[X
     *                                    ȂB
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get Monitoring object
     *
     * This operation returns an object implementing the Monitoring interface.
     * The Monitoring interface is one of the interfaces that each SDO
     * maintains. The interface is used to monitor the properties of an SDO.
     * See OMG SDO specification Section 2.3.7, "Monitoring Interface," on
     * page 2-35 for more details about the Monitoring interface.
     *
     * @return The Monitoring interface of an SDO.
     *
     * @exception InterfaceNotImplemented The target SDO has no Configuration
     *                                    interface.
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual SDOPackage::Monitoring_ptr get_monitoring()
      throw (CORBA::SystemException, 
	     SDOPackage::InterfaceNotImplemented, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] Organization Xg̎擾 
     *
     * SDO 0ȏ Organization (gD)ɏ邱ƂłB  SDO 
     * 1ȏ Organization ɏĂꍇÃIy[V͏
     * Organization ̃XgԂBSDO  ǂ Organization ɂĂȂ
     * ꍇɂ́ÃXgԂB
     *
     * @return SDO  Organization ̃XgB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [SDO interface] Get Organization list
     *
     * An SDO belongs to zero or more organizations. If the SDO belongs to one
     * or more organizations, this operation returns the list of organizations
     * that the SDO belongs to. An empty list is returned if the SDO does not
     * belong to any Organizations.
     *
     * @return The list of Organizations that the SDO belong to.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual SDOPackage::OrganizationList* get_organizations()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO Status Xg̎擾 
     *
     * ̃Iy[V SDO ̃Xe[^X\ NVList ԂB
     *
     * @return SDO ̃Xe[^XB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get SDO Status list
     *
     * This operation returns an NVlist describing the status of an SDO.
     *
     * @return The actual status of an SDO.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::NVList* get_status_list()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO Status ̎擾 
     *
     * This operation returns the value of the specified status parameter.
     * 
     * @param name SDO ̃Xe[^X`p[^B
     * 
     * @return w肳ꂽp[^̃Xe[^XlB
     * 
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InvalidParameter  "name"  null 邢݂͑ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [SDO interface] Get SDO Status
     *
     * @param name One of the parameters defining the "status" of an SDO.
     *
     * @return The value of the specified status parameter.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The parameter defined by "name" is null or
     *                             does not exist.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     *
     * @endif
     */
    virtual CORBA::Any* get_status(const char* name)
      throw (CORBA::SystemException, 
	     SDOPackage::InvalidParameter, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    //============================================================
    // Local interfaces
    //============================================================
    /*!
     * @if jp
     *
     * @brief [local interface] CX^X̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽCX^XԂB
     * 
     * @return CX^X
     * 
     * @else
     *
     * @brief [local interface] Get instance name
     * 
     * Return the instance name that has been set in ComponentProfile.
     * 
     * @return Instance name
     * 
     * @endif
     */
    const char* getInstanceName()
    {
      RTC_TRACE(("getInstanceName()"));
      return m_profile.instance_name;
    }
    
    /*!
     * @if jp
     *
     * @brief [local interface] CX^X̐ݒ
     * 
     * ComponentProfile Ɏw肳ꂽCX^Xݒ肷B
     * 
     * @param instance_name CX^X
     * 
     * @else
     *
     * @brief [local interface] Set instance name
     * 
     * Set the instance name specified in ComponentProfile.
     * 
     * @param instance_name Instance name
     * 
     * @endif
     */
    void setInstanceName(const char* instance_name);
    
    /*!
     * @if jp
     *
     * @brief [local interface] ^̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽ^ԂB
     * 
     * @return ^
     * 
     * @else
     *
     * @brief [local interface] Get type name
     * 
     * Get the type name has been set in ComponentProfile.
     * 
     * @return Type name
     * 
     * @endif
     */
    const char* getTypeName()
    {
      RTC_TRACE(("getTypeName()"));
      return m_profile.type_name;
    }
    
    /*!
     * @if jp
     *
     * @brief [local interface] Description ̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽ Description ԂB
     * 
     * @return Description
     * 
     * @else
     *
     * @brief [local interface] GetDescription
     * 
     * Get the Description has been set in ComponentProfile.
     * 
     * @return Description
     * 
     * @endif
     */
    const char* getDescription()
    {
      RTC_TRACE(("getDescription()"));
      return m_profile.description;
    }
    
    /*!
     * @if jp
     *
     * @brief [local interface] o[W̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽo[WԂB
     * 
     * @return o[W
     * 
     * @else
     *
     * @brief [local interface] Get version information
     * 
     * Get the version information that has been set in
     * ComponentProfile.
     * 
     * @return Version information
     * 
     * @endif
     */
    const char* getVersion()
    {
      RTC_TRACE(("getVersion()"));
      return m_profile.version;
    }
    
    /*!
     * @if jp
     *
     * @brief [local interface] x_[̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽx_[ԂB
     * 
     * @return x_[
     * 
     * @else
     *
     * @brief [local interface] Get vendor
     * 
     * Get the vendor information that has been set in ComponentProfile.
     * 
     * @return Vendor information
     * 
     * @endif
     */
    const char* getVendor()
    {
      RTC_TRACE(("getVendor()"));
      return m_profile.vendor;
    }
    
    /*!
     * @if jp
     *
     * @brief [local interface] JeS̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽJeSԂB
     * 
     * @return JeS
     * 
     * @else
     *
     * @brief [local interface] Get category information
     * 
     * Get the category information that has been set in ComponentProfile.
     * 
     * @return Category information
     * 
     * @endif
     */
    const char* getCategory()
    {
      RTC_TRACE(("getCategory()"));
      return m_profile.category;
    }
    
    /*!
     * @if jp
     *
     * @brief [local interface] Naming Server ̎擾
     * 
     * ݒ肳ꂽ Naming Server ԂB
     * 
     * @return Naming Server Xg
     * 
     * @else
     *
     * @brief [local interface] Get Naming Server information
     * 
     * Get Naming Server information that has been set.
     * 
     * @return Naming Server list
     * 
     * @endif
     */
    std::vector<std::string> getNamingNames();
    
    /*!
     * @if jp
     *
     * @brief [local interface] IuWFNgt@X̐ݒ
     * 
     * RTC  CORBA IuWFNgt@Xݒ肷B
     * 
     * @param rtobj IuWFNgt@X
     * 
     * @else
     *
     * @brief [local interface] Set the object reference
     * 
     * Set RTC's CORBA object reference.
     * 
     * @param rtobj The object reference
     * 
     * @endif
     */
    void setObjRef(const RTObject_ptr rtobj);
    
    /*!
     * @if jp
     *
     * @brief [local interface] IuWFNgt@X̎擾
     * 
     * ݒ肳ꂽ CORBA IuWFNgt@X擾B
     * 
     * @return IuWFNgt@X
     * 
     * @else
     *
     * @brief [local interface] Get the object reference
     * 
     * Get CORBA object reference that has been set
     * 
     * @return The object reference
     *
     * @endif
     */
    RTObject_ptr getObjRef() const;
    
    /*!
     * @if jp
     * 
     * @brief [local interface] RTC ̃vpeBݒ肷
     *
     * RTC ێׂvpeBݒ肷B^vpeB́A
     * ComponentProfile ɐݒ肳ׂȂ΂ȂȂB
     * ̃Iy[V͒ʏ RTC ۂ Manager 
     * Ă΂邱ƂӐ}ĂB
     * 
     * @param prop RTC ̃vpeB
     *
     * @else
     *
     * @brief [local interface] Set RTC property
     *
     * This operation sets the properties to the RTC. The given property
     * values should include information for ComponentProfile.
     * Generally, this operation is designed to be called from Manager, when
     * RTC is initialized
     *
     * @param prop Property for RTC.
     *
     * @endif
     */
    void setProperties(const coil::Properties& prop);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] RTC ̃vpeB擾
     *
     * RTC ێĂvpeBԂB
     * RTCvpeBȂꍇ͋̃vpeBԂB
     * 
     * @return RTC ̃vpeB
     *
     * @else
     *
     * @brief [local interface] Get RTC property
     *
     * This operation returns the properties of the RTC.
     * Empty property would be returned, if RTC has no property.
     *
     * @return Property for RTC.
     *
     * @endif
     */
    coil::Properties& getProperties();
    
    /*!
     * @if jp
     *
     * @brief RtBM[Vp[^̐ݒ
     * 
     * RtBM[Vp[^ƕϐoCh
     * \<VarType\>ƂăRtBM[Vp[^̃f[^^w肷B
     *
     * @param param_name RtBM[Vp[^
     * @param var RtBM[Vp[^i[pϐ
     * @param def_val RtBM[Vp[^ftHgl
     * @param trans RtBM[Vp[^ϊp֐
     *
     * @return ݒ茋(ݒ萬:trueCݒ莸s:false)
     * 
     * @else
     *
     * @brief Setup for configuration parameters
     * 
     * Bind configuration parameter to its variable.
     * Specify the data type of configuration parameter as \<VarType\>.
     *
     * @param param_name Configuration parameter name
     * @param var Variables to store configuration parameter
     * @param def_val Default value of configuration parameter
     * @param trans Function to transform configuration parameter type into 
     *        string format
     *
     * @return Setup result (Successful:true, Failed:false)
     * 
     * @endif
     */
    template <typename VarType>
    bool bindParameter(const char* param_name, VarType& var,
		       const char* def_val,
		       bool (*trans)(VarType&, const char*) = coil::stringTo)
    {
      RTC_TRACE(("bindParameter(%s (default: %s))", param_name, def_val));
      m_configsets.bindParameter(param_name, var, def_val, trans);
      return true;
    }
    
    /*!
     * @if jp
     *
     * @brief RtBM[Vp[^̍XV(IDw)
     * 
     * w肵ID̃RtBM[VZbgɐݒ肵lŁA
     * RtBM[Vp[^̒lXV
     *
     * @param config_set ݒΏۂ̃RtBM[VZbgID
     * 
     * @else
     *
     * @brief Update configuration parameters (by ID)
     * 
     * Update configuration parameter value by the value that 
     * set to a configuration set of specified ID.
     *
     * @param config_set The target configuration set's ID for setup
     * 
     * @endif
     */
    void updateParameters(const char* config_set);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] Port o^
     *
     * RTC ێPorto^B
     * Port OANZX\ɂ邽߂ɂ́ÃIy[Vɂ
     * o^ĂȂ΂ȂȂBo^ Port ͂ RTC ɂ
     * PortProfile.name ɂʂBāAPort  RTC ɂāA
     * j[N PortProfile.name Ȃ΂ȂȂB
     * o^ꂽ Port ͓œK؂ɃANeBuꂽA̎QƂ
     * IuWFNgQƂXgɕۑB
     * 
     * @param port RTC ɓo^ Port
     * @return o^(o^:trueCo^s:false)
     *
     * @else
     *
     * @brief [local interface] Register Port
     *
     * This operation registers a Port held by this RTC.
     * In order to enable access to the Port from outside of RTC, the Port
     * must be registered by this operation. The Port that is registered by
     * this operation would be identified by PortProfile.name in the inside of
     * RTC. Therefore, the Port should have unique PortProfile.name in the RTC.
     * The registering Port would be activated properly, and the reference
     * and the object reference would be stored in lists in RTC.
     *
     * @param port Port which is registered to the RTC
     * @return Register result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool addPort(PortBase& port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port o^
     *
     * RTC ێPorto^B
     * Port OANZX\ɂ邽߂ɂ́ÃIy[Vɂ
     * o^ĂȂ΂ȂȂBo^ Port ͂ RTC ɂ
     * PortProfile.name ɂʂBāAPort  RTC ɂāA
     * j[N PortProfile.name Ȃ΂ȂȂB
     * o^ꂽ Port ͓œK؂ɃANeBuꂽA̎QƂ
     * IuWFNgQƂXgɕۑB
     * 
     * @param port RTC ɓo^ Port
     * @return o^(o^:trueCo^s:false)
     *
     * @else
     *
     * @brief [local interface] Register Port
     *
     * This operation registers a Port held by this RTC.
     * In order to enable access to the Port from outside of RTC, the Port
     * must be registered by this operation. The Port that is registered by
     * this operation would be identified by PortProfile.name in the inside of
     * RTC. Therefore, the Port should have unique PortProfile.name in the RTC.
     * The registering Port would be activated properly, and the reference
     * and the object reference would be stored in lists in RTC.
     *
     * @param port Port which is registered to the RTC
     * @return Register result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool addPort(PortService_ptr port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port o^
     *
     * RTC ێPorto^B
     * Port OANZX\ɂ邽߂ɂ́ÃIy[Vɂ
     * o^ĂȂ΂ȂȂBo^ Port ͂ RTC ɂ
     * PortProfile.name ɂʂBāAPort  RTC ɂāA
     * j[N PortProfile.name Ȃ΂ȂȂB
     * o^ꂽ Port ͓œK؂ɃANeBuꂽA̎QƂ
     * IuWFNgQƂXgɕۑB
     * 
     * @param port RTC ɓo^ Port
     * @return o^(o^:trueCo^s:false)
     *
     * @else
     *
     * @brief [local interface] Register Port
     *
     * This operation registers a Port held by this RTC.
     * In order to enable access to the Port from outside of RTC, the Port
     * must be registered by this operation. The Port that is registered by
     * this operation would be identified by PortProfile.name in the inside of
     * RTC. Therefore, the Port should have unique PortProfile.name in the RTC.
     * The registering Port would be activated properly, and the reference
     * and the object reference would be stored in lists in RTC.
     *
     * @param port Port which is registered to the RTC
     * @return Register result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool addPort(CorbaPort& port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port o^
     *
     * RTC ێPorto^B
     * Port OANZX\ɂ邽߂ɂ́ÃIy[Vɂ
     * o^ĂȂ΂ȂȂBo^ Port ͂ RTC ɂ
     * PortProfile.name ɂʂBāAPort  RTC ɂāA
     * j[N PortProfile.name Ȃ΂ȂȂB
     * o^ꂽ Port ͓œK؂ɃANeBuꂽA̎QƂ
     * IuWFNgQƂXgɕۑB
     * 
     * @param port RTC ɓo^ Port
     *
     * @else
     *
     * @brief [local interface] Register Port
     *
     * This operation registers a Port held by this RTC.
     * In order to enable access to the Port from outside of RTC, the Port
     * must be registered by this operation. The Port that is registered by
     * this operation would be identified by PortProfile.name in the inside of
     * RTC. Therefore, the Port should have unique PortProfile.name in the RTC.
     * The registering Port would be activated properly, and the reference
     * and the object reference would be stored in lists in RTC.
     *
     * @param port Port which is registered to the RTC
     *
     * @endif
     */
    void registerPort(PortBase& port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port o^
     *
     * RTC ێPorto^B
     * Port OANZX\ɂ邽߂ɂ́ÃIy[Vɂ
     * o^ĂȂ΂ȂȂBo^ Port ͂ RTC ɂ
     * PortProfile.name ɂʂBāAPort  RTC ɂāA
     * j[N PortProfile.name Ȃ΂ȂȂB
     * o^ꂽ Port ͓œK؂ɃANeBuꂽA̎QƂ
     * IuWFNgQƂXgɕۑB
     * 
     * @param port RTC ɓo^ Port
     *
     * @else
     *
     * @brief [local interface] Register Port
     *
     * This operation registers a Port held by this RTC.
     * In order to enable access to the Port from outside of RTC, the Port
     * must be registered by this operation. The Port that is registered by
     * this operation would be identified by PortProfile.name in the inside of
     * RTC. Therefore, the Port should have unique PortProfile.name in the RTC.
     * The registering Port would be activated properly, and the reference
     * and the object reference would be stored in lists in RTC.
     *
     * @param port Port which is registered to the RTC
     *
     * @endif
     */
    void registerPort(PortService_ptr port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port o^
     *
     * RTC ێPorto^B
     * Port OANZX\ɂ邽߂ɂ́ÃIy[Vɂ
     * o^ĂȂ΂ȂȂBo^ Port ͂ RTC ɂ
     * PortProfile.name ɂʂBāAPort  RTC ɂāA
     * j[N PortProfile.name Ȃ΂ȂȂB
     * o^ꂽ Port ͓œK؂ɃANeBuꂽA̎QƂ
     * IuWFNgQƂXgɕۑB
     * 
     * @param port RTC ɓo^ Port
     *
     * @else
     *
     * @brief [local interface] Register Port
     *
     * This operation registers a Port held by this RTC.
     * In order to enable access to the Port from outside of RTC, the Port
     * must be registered by this operation. The Port that is registered by
     * this operation would be identified by PortProfile.name in the inside of
     * RTC. Therefore, the Port should have unique PortProfile.name in the RTC.
     * The registering Port would be activated properly, and the reference
     * and the object reference would be stored in lists in RTC.
     *
     * @param port Port which is registered to the RTC
     *
     * @endif
     */
    void registerPort(CorbaPort& port);

    /*!
     * @if jp
     * 
     * @brief [local interface] DataInPort o^
     *
     * RTC ێ DataInPort o^B
     * Port ̃vpeBɃf[^|[gł邱("port.dataport")A
     * TCPgp邱("tcp_any")ݒ肷ƂƂɁA DataInPort 
     * CX^X𐶐Ao^B
     * 
     * @param name port 
     * @param inport o^Ώ DataInPort
     * @return o^(o^:trueCo^s:false)
     *
     * @else
     * 
     * @brief [local interface] Register DataInPort
     *
     * This operation registers DataInPort held by this RTC.
     * Set "port.dataport" and "tcp_any" to property of Port, and
     * create instances of DataInPort and register it.
     * 
     * @param name Port name
     * @param inport DataInPort which is registered to the RTC
     * @return Register result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool addInPort(const char* name, InPortBase& inport);
    /*!
     * @if jp
     * 
     * @brief [local interface] DataInPort o^
     *
     * RTC ێ DataInPort o^B
     * Port ̃vpeBɃf[^|[gł邱("port.dataport")A
     * TCPgp邱("tcp_any")ݒ肷ƂƂɁA DataInPort 
     * CX^X𐶐Ao^B
     * 
     * @param name port 
     * @param inport o^Ώ DataInPort
     *
     * @else
     * 
     * @brief [local interface] Register DataInPort
     *
     * This operation registers DataInPort held by this RTC.
     * Set "port.dataport" and "tcp_any" to property of Port, and
     * create instances of DataInPort and register it.
     * 
     * @param name Port name
     * @param inport DataInPort which is registered to the RTC
     *
     * @endif
     */
    void registerInPort(const char* name, InPortBase& inport);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] DataOutPort o^
     *
     * RTC ێ DataOutPorto^B
     * Port ̃vpeBɃf[^|[gł邱("port.dataport")A
     * TCPgp邱("tcp_any")ݒ肷ƂƂɁA DataOutPort 
     * CX^X𐶐Ao^B
     * 
     * @param name port 
     * @param outport o^Ώ DataOutPort
     * @return o^(o^:trueCo^s:false)
     *
     * @else
     * 
     * @brief [local interface] Register DataOutPort
     *
     * This operation registers DataOutPor held by this RTC.
     * Set "port.dataport" and "tcp_any" to property of Port, and then
     * create instances of DataOutPort and register it.
     * 
     * @param name Port name
     * @param outport DataOutPort which is registered to the RTC
     * @return Register result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool addOutPort(const char* name, OutPortBase& outport);
    /*!
     * @if jp
     * 
     * @brief [local interface] DataOutPort o^
     *
     * RTC ێ DataOutPorto^B
     * Port ̃vpeBɃf[^|[gł邱("port.dataport")A
     * TCPgp邱("tcp_any")ݒ肷ƂƂɁA DataOutPort 
     * CX^X𐶐Ao^B
     * 
     * @param name port 
     * @param outport o^Ώ DataOutPort
     *
     * @else
     * 
     * @brief [local interface] Register DataOutPort
     *
     * This operation registers DataOutPor held by this RTC.
     * Set "port.dataport" and "tcp_any" to property of Port, and then
     * create instances of DataOutPort and register it.
     * 
     * @param name Port name
     * @param outport DataOutPort which is registered to the RTC
     *
     * @endif
     */
    void registerOutPort(const char* name, OutPortBase& outport);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] InPort ̓o^폜
     *
     * RTC ێInPort̓o^폜B
     * 
     * @param port 폜Ώ Port
     * @return 폜(폜:trueC폜s:false)
     *
     * @else
     *
     * @brief [local interface] Unregister InPort
     *
     * This operation unregisters a InPort held by this RTC.
     *
     * @param port Port which is unregistered
     * @return Unregister result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool removeInPort(InPortBase& port);

    /*!
     * @if jp
     * 
     * @brief [local interface] OutPort ̓o^폜
     *
     * RTC ێOutPort̓o^폜B
     * 
     * @param port 폜Ώ Port
     * @return 폜(폜:trueC폜s:false)
     *
     * @else
     *
     * @brief [local interface] Unregister OutPort
     *
     * This operation unregisters a OutPort held by this RTC.
     *
     * @param port Port which is unregistered
     * @return Unregister result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool removeOutPort(OutPortBase& port);

    /*!
     * @if jp
     * 
     * @brief [local interface] Port ̓o^폜
     *
     * RTC ێPort̓o^폜B
     * 
     * @param port 폜Ώ Port
     * @return 폜(폜:trueC폜s:false)
     *
     * @else
     *
     * @brief [local interface] Unregister Port
     *
     * This operation unregisters a Port held by this RTC.
     *
     * @param port Port which is unregistered
     * @return Unregister result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool removePort(PortBase& port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port ̓o^폜
     *
     * RTC ێPort̓o^폜B
     * 
     * @param port 폜Ώ Port
     * @return 폜(폜:trueC폜s:false)
     *
     * @else
     *
     * @brief [local interface] Unregister Port
     *
     * This operation unregisters a Port held by this RTC.
     *
     * @param port Port which is unregistered
     * @return Unregister result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool removePort(PortService_ptr port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port ̓o^폜
     *
     * RTC ێPort̓o^폜B
     * 
     * @param port 폜Ώ Port
     * @return 폜(폜:trueC폜s:false)
     *
     * @else
     *
     * @brief [local interface] Unregister Port
     *
     * This operation unregisters a Port held by this RTC.
     *
     * @param port Port which is unregistered
     * @return Unregister result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool removePort(CorbaPort& port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port ̓o^폜
     *
     * RTC ێPort̓o^폜B
     * 
     * @param port 폜Ώ Port
     *
     * @else
     *
     * @brief [local interface] Unregister Port
     *
     * This operation unregisters a Port held by this RTC.
     *
     * @param port Port which is unregistered
     *
     * @endif
     */
    void deletePort(PortBase& port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port ̓o^폜
     *
     * RTC ێPort̓o^폜B
     * 
     * @param port 폜Ώ Port
     *
     * @else
     *
     * @brief [local interface] Unregister Port
     *
     * This operation unregisters a Port held by this RTC.
     *
     * @param port Port which is unregistered
     *
     * @endif
     */
    void deletePort(PortService_ptr port);
    /*!
     * @if jp
     * 
     * @brief [local interface] Port ̓o^폜
     *
     * RTC ێPort̓o^폜B
     * 
     * @param port 폜Ώ Port
     *
     * @else
     *
     * @brief [local interface] Unregister Port
     *
     * This operation unregisters a Port held by this RTC.
     *
     * @param port Port which is unregistered
     *
     * @endif
     */
    void deletePort(CorbaPort& port);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] Owɂ Port ̓o^폜
     *
     * ̂w肵 RTC ێPort̓o^폜B
     * 
     * @param port_name 폜Ώ Port 
     *
     * @else
     * 
     * @brief [local interface] Delete Port by specifying its name
     *
     * Delete Port which RTC has by specifying its name.
     * 
     * @param port_name Name of Port which is deleted
     *
     * @endif
     */
    void deletePortByName(const char* port_name);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] sReLXg擾
     *
     * get_context() Ɠ@\̃[JŁBႢ͂ȂB
     * ̊֐͈ȉ̊֐ŌĂ΂邱ƂOƂĂB
     *
     * - onStartup()
     * - onShutdown()
     * - onActivated()
     * - onDeactivated()
     * - onExecute()
     * - onAborting()
     * - onError()
     * - onReset()
     * - onStateUpdate()
     * - onRateChanged()
     *
     * ̊֐̈͂̊֐̈ UniquieID exec_handle łȂ
     * ΂ȂȂB
     *
     * @param ec_id L֐̑1 exec_handle nKvB
     *
     * @else
     * 
     * @brief [local interface] Getting current execution context
     *
     * This function is the local version of get_context(). completely
     * same as get_context() function. This function is assumed to be
     * called from the following functions.
     *
     * - onStartup()
     * - onShutdown()
     * - onActivated()
     * - onDeactivated()
     * - onExecute()
     * - onAborting()
     * - onError()
     * - onReset()
     * - onStateUpdate()
     * - onRateChanged()
     *
     * The argument of this function should be the first argument
     * (UniqueId ec_id) of the above functions.
     *
     * @param ec_id The above functions' first argument "exec_handle."
     *
     * @endif
     */
    ExecutionContext_ptr getExecutionContext(RTC::UniqueId ec_id);

    /*!
     * @if jp
     * 
     * @brief [local interface] sReLXg̎s[g擾
     *
     * ݎs̎sReLXg̎s[g擾BsReL
     * XgKindPERIODICȊȌꍇ͖̓`łB̊֐͈
     * ̊֐ŌĂ΂邱ƂOƂĂB
     *
     * - onStartup()
     * - onShutdown()
     * - onActivated()
     * - onDeactivated()
     * - onExecute()
     * - onAborting()
     * - onError()
     * - onReset()
     * - onStateUpdate()
     * - onRateChanged()
     *
     * ̊֐̈͂̊֐̈ UniquieID exec_handle łȂ
     * ΂ȂȂB
     *
     * @param ec_id L֐̑1 exec_handle nKvB
     *
     * @else
     * 
     * @brief [local interface] Getting current context' execution rate
     *
     * This function returns current execution rate in this
     * context. If this context's kind is not PERIODC, behavior is not
     * defined. This function is assumed to be called from the
     * following functions.
     *
     * - onStartup()
     * - onShutdown()
     * - onActivated()
     * - onDeactivated()
     * - onExecute()
     * - onAborting()
     * - onError()
     * - onReset()
     * - onStateUpdate()
     * - onRateChanged()
     *
     * The argument of this function should be the first argument
     * (UniqueId ec_id) of the above functions.
     *
     * @param ec_id The above functions' first argument "exec_handle."
     *
     * @endif
     */
    double getExecutionRate(RTC::UniqueId ec_id);

    /*!
     * @if jp
     * 
     * @brief [local interface] sReLXg̎s[gݒ肷
     *
     * ݎs̎sReLXg̎s[gݒ肷BsReL
     * XgKindPERIODICȊȌꍇ͖̓`łB̊֐͈
     * ̊֐ŌĂ΂邱ƂOƂĂB
     *
     * - onStartup()
     * - onShutdown()
     * - onActivated()
     * - onDeactivated()
     * - onExecute()
     * - onAborting()
     * - onError()
     * - onReset()
     * - onStateUpdate()
     * - onRateChanged()
     *
     * ̊֐̈͂̊֐̈ UniquieID exec_handle łȂ
     * ΂ȂȂB
     *
     * @param ec_id L֐̑1 exec_handle nKvB
     * @param rate s[g [Hz] ŗ^
     *
     * @else
     * 
     * @brief [local interface] Setting current context' execution rate
     *
     * This function sets a execution rate in the context. If this
     * context's kind is not PERIODC, behavior is not defined. This
     * function is assumed to be called from the following functions.
     *
     * - onStartup()
     * - onShutdown()
     * - onActivated()
     * - onDeactivated()
     * - onExecute()
     * - onAborting()
     * - onError()
     * - onReset()
     * - onStateUpdate()
     * - onRateChanged()
     *
     * The argument of this function should be the first argument
     * (UniqueId ec_id) of the above functions.
     *
     * @param ec_id The above functions' first argument "exec_handle."
     * @param rate Execution rate in [Hz].
     *
     * @endif
     */
    ReturnCode_t setExecutionRate(RTC::UniqueId ec_id, double rate);

    /*!
     * @if jp
     * 
     * @brief [local interface] sReLXg̏L𒲂ׂ
     *
     * ݎs̎sReLXg̏L𒲂ׂB̊֐͈ȉ̊
     * ŌĂ΂邱ƂOƂĂB
     *
     * - onStartup()
     * - onShutdown()
     * - onActivated()
     * - onDeactivated()
     * - onExecute()
     * - onAborting()
     * - onError()
     * - onReset()
     * - onStateUpdate()
     * - onRateChanged()
     *
     * ̊֐̈͂̊֐̈ UniquieID exec_handle łȂ
     * ΂ȂȂB
     *
     * @param ec_id L֐̑1 exec_handle nKvB
     * @return true: g̎sReLXgAfalse: ̎sReLXg
     *
     * @else
     * 
     * @brief [local interface] Checking if the current context is own context
     *
     * This function checks if the current context is own execution
     * context. This function is assumed to be called from the
     * following functions.
     *
     * - onStartup()
     * - onShutdown()
     * - onActivated()
     * - onDeactivated()
     * - onExecute()
     * - onAborting()
     * - onError()
     * - onReset()
     * - onStateUpdate()
     * - onRateChanged()
     *
     * The argument of this function should be the first argument
     * (UniqueId ec_id) of the above functions.
     *
     * @param ec_id The above functions' first argument "exec_handle."
     * @return true: Own context, false: other's context
     *
     * @endif
     */
    bool isOwnExecutionContext(RTC::UniqueId ec_id);

    /*!
     * @if jp
     * 
     * @brief [local interface] Ԃ Inactive ɑJڂ
     *
     * Ԃ Active  Inactive ɑJڂB̊֐͈ȉ̊
     * ŌĂ΂邱ƂOƂĂB
     *
     * - onActivated()
     * - onExecute()
     * - onStateUpdate()
     *
     * ̊֐̈͏L̊֐̈ UniquieID exec_handle łȂ
     * ΂ȂȂB
     *
     * @param ec_id L֐̑1 exec_handle nKvB
     * @return ^[R[h
     *
     * @else
     * 
     * @brief [local interface] Make transition to Inactive state
     *
     * This function makes transition from Active to Inactive
     * state. This function is assumed to be called from the following
     * functions.
     *
     * - onActivated()
     * - onExecute()
     * - onStateUpdate()
     *
     * The argument of this function should be the first argument
     * (UniqueId ec_id) of the above function.
     *
     * @param ec_id The above functions' first argument "exec_handle."
     * @return Return code
     *
     * @endif
     */
    ReturnCode_t deactivate(RTC::UniqueId ec_id);

    /*!
     * @if jp
     * 
     * @brief [local interface] Ԃ Active ɑJڂ
     *
     * Ԃ Inactive  Active ɑJڂB̊֐͈ȉ̊
     * ŌĂ΂邱ƂOƂĂB
     *
     * - onStartup()
     * - onDeactivated()
     *
     * ̊֐̈͏L̊֐̈ UniquieID exec_handle łȂ
     * ΂ȂȂB
     *
     * @param ec_id L֐̑1 exec_handle nKvB
     * @return ^[R[h
     *
     * @else
     * 
     * @brief [local interface] Make transition to Active state
     *
     * This function makes transition from Inactive to Active
     * state. This function is assumed to be called from the following
     * functions.
     *
     * - onStartup()
     * - onDeactivated()
     *
     * The argument of this function should be the first argument
     * (UniqueId ec_id) of the above function.
     *
     * @param ec_id The above functions' first argument "exec_handle."
     * @return Return code
     *
     * @endif
     */
    ReturnCode_t activate(RTC::UniqueId ec_id);

    /*!
     * @if jp
     * 
     * @brief [local interface] ԂZbg Inactive ɑJڂ
     *
     * Ԃ Error  Inactive ɑJڂB̊֐͈ȉ̊
     * ŌĂ΂邱ƂOƂĂB
     *
     * - onError()
     *
     * ̊֐̈͏L̊֐̈ UniquieID exec_handle łȂ
     * ΂ȂȂB
     *
     * @param ec_id L֐̑1 exec_handle nKvB
     * @return ^[R[h
     *
     * @else
     * 
     * @brief [local interface] Resetting and go to Inactive state
     *
     * This function reset RTC and makes transition from Error to Inactive
     * state. This function is assumed to be called from the following
     * functions.
     *
     * - onError()
     *
     * The argument of this function should be the first argument
     * (UniqueId ec_id) of the above function.
     *
     * @param ec_id The above functions' first argument "exec_handle."
     * @return Return code
     *
     * @endif
     */
    ReturnCode_t reset(RTC::UniqueId ec_id);

    /*!
     * @if jp
     * @brief [local interface] SDO service provider Zbg
     * @else
     * @brief [local interface] Set a SDO service provider
     * @endif
     */
    bool addSdoServiceProvider(const SDOPackage::ServiceProfile& prof,
                               SdoServiceProviderBase* provider);

    /*!
     * @if jp
     * @brief [local interface] SDO service provider 폜
     * @else
     * @brief [local interface] Remove a SDO service provider
     * @endif
     */
    bool removeSdoServiceProvider(const char* id);

    /*!
     * @if jp
     * @brief [local interface] SDO service provider Zbg
     * @else
     * @brief [local interface] Set a SDO service provider
     * @endif
     */
    bool addSdoServiceConsumer(const SDOPackage::ServiceProfile& prof);

    /*!
     * @if jp
     * @brief [local interface] SDO service provider 폜
     * @else
     * @brief [local interface] Remove a SDO service provider
     * @endif
     */
    bool removeSdoServiceConsumer(const char* id);

    /*!
     * @if jp
     *
     * @brief S InPort ̃f[^ǂݍށB
     *
     * RTC ێSĂ InPort ̃f[^ǂݍށB
     *
     * @return ǂݍ݌(S|[g̓ǂݍݐ:trueCs:false)
     *
     * @else
     *
     * @brief Readout the value from All InPorts.
     *
     * This operation read the value from all InPort
     * registered in the RTC.
     *
     * @return result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool readAll();

    /*!
     * @if jp
     *
     * @brief S OutPort write()\bhR[B
     *
     * RTC ێSĂ OutPort write()\bhR[B
     *
     * @return ǂݍ݌(S|[gւ̏ݐ:trueCs:false)
     *
     * @else
     *
     * @brief The write() method of all OutPort is called. 
     *
     * This operation call the write() method of all OutPort
     * registered in the RTC.
     *
     * @return result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool writeAll();

    /*!
     * @if jp
     *
     * @brief onExecute()sOłreadAll()\bȟďoL܂͖ɂB
     *
     * ̃\bhp[^trueƂČĂԎɂAonExecute()sO
     * readAll()ďo悤ɂȂB
     * p[^falsȅꍇ́AreadAll()ďo𖳌ɂB
     *
     * @param read(default:true) 
     *        (readAll()\bhďo:true, readAll()\bhďoȂ:false)
     *
     * @param completion(default:false) 
     *    readAll()ɂāAǂꂩ̈InPortread()sĂSĂ
     *    InPortread()Ăяo:true, readAll()ɂāAǂꂩ̈
     *    InPortread()sꍇAfalseŔ:false
     *
     * @else
     *
     * @brief Set whether to execute the readAll() method. 
     *
     * Set whether to execute the readAll() method. 
     *
     * @param read(default:true)
     *        (readAll() is called:true, readAll() isn't called:false)
     *
     * @param completion(default:false)
     *     All InPort::read() calls are completed.:true,
     *     If one InPort::read() is False, return false.:false
     *
     * @param completion(default:false)
     *
     * @endif
     */
    void setReadAll(bool read=true, bool completion=false);

    /*!
     * @if jp
     *
     * @brief onExecute()swriteAll()\bȟďoL܂͖ɂB
     *
     * ̃\bhp[^trueƂČĂԎɂAonExecute()s
     * writeAll()ďo悤ɂȂB
     * p[^falsȅꍇ́AwriteAll()ďo𖳌ɂB
     *
     * @param write(default:true) 
     *        (writeAll()\bhďo:true, writeAll()\bhďo
     *        Ȃ:false)
     *
     * @param completion(default:false) 
     *    writeAll()ɂāAǂꂩ̈OutPortwrite()sĂS
     *    ĂOutPortwrite()Ăяos:true, writeAll()ɂāA
     *    ꂩ̈OutPortwrite()sꍇAfalseŔ
     *    :false
     *
     * @else
     *
     * @brief Set whether to execute the writeAll() method. 
     *
     * Set whether to execute the writeAll() method. 
     *
     * @param write(default:true)
     *        (writeAll() is called:true, writeAll() isn't called:false)
     *
     * @param completion(default:false)
     *     All OutPort::write() calls are completed.:true,
     *     If one OutPort::write() is False, return false.:false
     *
     * @endif
     */
    void setWriteAll(bool write=true, bool completion=false);


    /*!
     * @if jp
     *
     * @brief S Port ̓o^폜
     *
     * RTC ێSĂ Port 폜B
     *
     * @else
     *
     * @brief Unregister All Ports
     *
     * This operation deactivates the all Ports and deletes the all Port's
     * registrations in the RTC
     *
     * @endif
     */
    void finalizePorts();


    /*!
     * @if jp
     *
     * @brief ExecutionContextBaseXg̓o^폜 
     *
     * @else
     *
     * @brief The ExecutionContextBase list is deleted. 
     *
     * @endif
     */
    void finalizeContexts();


    /*!
     * @if jp
     * @brief PreComponentActionListener Xiǉ
     *
     * ComponentAction ֐̌ĂяoÕCxgɊ֘Ae탊
     * Xiݒ肷B
     *
     * ݒł郊Xĩ^CvƃR[obNCxg͈ȉ̒ʂ
     *
     * - PRE_ON_INITIALIZE:    onInitialize O
     * - PRE_ON_FINALIZE:      onFinalize O
     * - PRE_ON_STARTUP:       onStartup O
     * - PRE_ON_SHUTDOWN:      onShutdown O
     * - PRE_ON_ACTIVATED:     onActivated O
     * - PRE_ON_DEACTIVATED:   onDeactivated O
     * - PRE_ON_ABORTED:       onAborted O
     * - PRE_ON_ERROR:         onError O
     * - PRE_ON_RESET:         onReset O
     * - PRE_ON_EXECUTE:       onExecute O
     * - PRE_ON_STATE_UPDATE:  onStateUpdate O
     *
     * Xi PreComponentActionListener pAȉ̃VOj`
     * operator() ĂKvB
     *
     * PreComponentActionListener::operator()(UniqueId ec_id)
     *
     * ftHgł́Å֐ɗ^XiIuWFNg̏L
     * RTObjectɈڂARTObject̎́A
     * removePreComponentActionListener() ɂ폜ɎIɉ̂B
     * XiIuWFNg̏LĂяoňێꍇ́A3
     *  false w肵AIȉ̂}邱ƂłB
     *
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     * @param autoclean XiIuWFNg̎Îsǂ̃tO
     *
     * @else
     * @brief Adding PreComponentAction type listener
     *
     * This operation adds certain listeners related to ComponentActions
     * pre events.
     * The following listener types are available.
     *
     * - PRE_ON_INITIALIZE:    before onInitialize
     * - PRE_ON_FINALIZE:      before onFinalize
     * - PRE_ON_STARTUP:       before onStartup
     * - PRE_ON_SHUTDOWN:      before onShutdown
     * - PRE_ON_ACTIVATED:     before onActivated
     * - PRE_ON_DEACTIVATED:   before onDeactivated
     * - PRE_ON_ABORTED:       before onAborted
     * - PRE_ON_ERROR:         before onError
     * - PRE_ON_RESET:         before onReset
     * - PRE_ON_EXECUTE:       before onExecute
     * - PRE_ON_STATE_UPDATE:  before onStateUpdate
     *
     * Listeners should have the following function operator().
     *
     * PreComponentActionListener::operator()(UniqueId ec_id)
     *
     * The ownership of the given listener object is transferred to
     * this RTObject object in default.  The given listener object will
     * be destroied automatically in the RTObject's dtor or if the
     * listener is deleted by removePreComponentActionListener() function.
     * If you want to keep ownership of the listener object, give
     * "false" value to 3rd argument to inhibit automatic destruction.
     *
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     * @param autoclean A flag for automatic listener destruction
     *
     * @endif
     */
    typedef PreComponentActionListener PreCompActionListener;
    typedef PreComponentActionListenerType PreCompActionListenerType;
    void 
    addPreComponentActionListener(PreComponentActionListenerType listener_type,
                                  PreComponentActionListener* listener,
                                  bool autoclean = true);


    template <class Listener>
    PreComponentActionListener*
    addPreComponentActionListener(PreCompActionListenerType listener_type,
                                   Listener& obj,
                                   void (Listener::*memfunc)(UniqueId ec_id))
    {
      class Noname
        : public PreComponentActionListener
      {
      public:
        Noname(Listener& obj, void (Listener::*memfunc)(UniqueId))
          : m_obj(obj), m_memfunc(memfunc)
        {
        }
        void operator()(UniqueId ec_id)
        {
          (m_obj.*m_memfunc)(ec_id);
        }
      private:
        Listener& m_obj;
        typedef void (Listener::*Memfunc)(UniqueId ec_id);
        Memfunc m_memfunc;
      };
      Noname* listener(new Noname(obj, memfunc));
      addPreComponentActionListener(listener_type, listener, true);
      return listener;
    }

    /*!
     * @if jp
     * @brief PreComponentActionListener Xi폜
     *
     * ݒ肵e탊Xi폜B
     * 
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     *
     * @else
     * @brief Removing PreComponentAction type listener
     *
     * This operation removes a specified listener.
     *     
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     *
     * @endif
     */
    void 
    removePreComponentActionListener(
                                 PreComponentActionListenerType listener_type,
                                 PreComponentActionListener* listener);


    /*!
     * @if jp
     * @brief PostComponentActionListener Xiǉ
     *
     * ComponentAction ֐̌ĂяõCxgɊ֘Ae탊
     * Xiݒ肷B
     *
     * ݒł郊Xĩ^CvƃR[obNCxg͈ȉ̒ʂ
     *
     * - POST_ON_INITIALIZE:    onInitialize 
     * - POST_ON_FINALIZE:      onFinalize 
     * - POST_ON_STARTUP:       onStartup 
     * - POST_ON_SHUTDOWN:      onShutdown 
     * - POST_ON_ACTIVATED:     onActivated 
     * - POST_ON_DEACTIVATED:   onDeactivated 
     * - POST_ON_ABORTED:       onAborted 
     * - POST_ON_ERROR:         onError 
     * - POST_ON_RESET:         onReset 
     * - POST_ON_EXECUTE:       onExecute 
     * - POST_ON_STATE_UPDATE:  onStateUpdate 
     *
     * Xi PostComponentActionListener pAȉ̃VOj`
     * operator() ĂKvB
     *
     * PostComponentActionListener::operator()(UniqueId ec_id, ReturnCode_t ret)
     *
     * ftHgł́Å֐ɗ^XiIuWFNg̏L
     * RTObjectɈڂARTObject̎́A
     * removePostComponentActionListener() ɂ폜ɎIɉ̂B
     * XiIuWFNg̏LĂяoňێꍇ́A3
     *  false w肵AIȉ̂}邱ƂłB
     *
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     * @param autoclean XiIuWFNg̎Îsǂ̃tO
     *
     * @else
     * @brief Adding PostComponentAction type listener
     *
     * This operation adds certain listeners related to ComponentActions
     * post events.
     * The following listener types are available.
     *
     * - POST_ON_INITIALIZE:    after onInitialize
     * - POST_ON_FINALIZE:      after onFinalize
     * - POST_ON_STARTUP:       after onStartup
     * - POST_ON_SHUTDOWN:      after onShutdown
     * - POST_ON_ACTIVATED:     after onActivated
     * - POST_ON_DEACTIVATED:   after onDeactivated
     * - POST_ON_ABORTED:       after onAborted
     * - POST_ON_ERROR:         after onError
     * - POST_ON_RESET:         after onReset
     * - POST_ON_EXECUTE:       after onExecute
     * - POST_ON_STATE_UPDATE:  after onStateUpdate
     *
     * Listeners should have the following function operator().
     *
     * PostComponentActionListener::operator()(UniqueId ec_id, ReturnCode_t ret)
     *
     * The ownership of the given listener object is transferred to
     * this RTObject object in default.  The given listener object will
     * be destroied automatically in the RTObject's dtor or if the
     * listener is deleted by removePostComponentActionListener() function.
     * If you want to keep ownership of the listener object, give
     * "false" value to 3rd argument to inhibit automatic destruction.
     *
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     * @param autoclean A flag for automatic listener destruction
     *
     * @endif
     */
    typedef PostComponentActionListener PostCompActionListener;
    typedef PostComponentActionListenerType PostCompActionListenerType;
    void 
    addPostComponentActionListener(
                               PostComponentActionListenerType listener_type,
                               PostComponentActionListener* listener,
                               bool autoclean = true);

    template <class Listener>
    PostComponentActionListener*
    addPostComponentActionListener(PostCompActionListenerType listener_type,
                                   Listener& obj,
                                   void (Listener::*memfunc)(UniqueId ec_id,
                                                             ReturnCode_t ret))
    {
      class Noname
        : public PostComponentActionListener
      {
      public:
        Noname(Listener& obj, void (Listener::*memfunc)(UniqueId, ReturnCode_t))
          : m_obj(obj), m_memfunc(memfunc)
        {
        }
        void operator()(UniqueId ec_id, ReturnCode_t ret)
        {
          (m_obj.*m_memfunc)(ec_id, ret);
        }
      private:
        Listener& m_obj;
        typedef void (Listener::*Memfunc)(UniqueId ec_id, ReturnCode_t ret);
        Memfunc m_memfunc;
      };
      Noname* listener(new Noname(obj, memfunc));
      addPostComponentActionListener(listener_type, listener, true);
      return listener;
    }

    /*!
     * @if jp
     * @brief PostComponentActionListener Xi폜
     *
     * ݒ肵e탊Xi폜B
     * 
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     *
     * @else
     * @brief Removing PostComponentAction type listener
     *
     * This operation removes a specified listener.
     *     
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     *
     * @endif
     */
    void 
    removePostComponentActionListener(
                                  PostComponentActionListenerType listener_type,
                                  PostComponentActionListener* listener);



    /*!
     * @if jp
     * @brief PortActionListener Xiǉ
     *
     * Port̒ǉA폜ɃR[obNe탊Xiݒ肷B
     *
     * ݒł郊Xĩ^CvƃR[obNCxg͈ȉ̒ʂ
     *
     * - ADD_PORT:    Portǉ
     * - REMOVE_PORT: Port폜
     *
     * Xi PortActionListener pAȉ̃VOj`
     * operator() ĂKvB
     *
     * PortActionListener::operator()(PortProfile& pprof)
     *
     * ftHgł́Å֐ɗ^XiIuWFNg̏L
     * RTObjectɈڂARTObject̎́A
     * removePortActionListener() ɂ폜ɎIɉ̂B
     * XiIuWFNg̏LĂяoňێꍇ́A3
     *  false w肵AIȉ̂}邱ƂłB
     *
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     * @param autoclean XiIuWFNg̎Îsǂ̃tO
     *
     * @else
     * @brief Adding PortAction type listener
     *
     * This operation adds certain listeners related to ComponentActions
     * post events.
     * The following listener types are available.
     *
     * - ADD_PORT:    At adding Port
     * - REMOVE_PORT: At removing Port
     *
     * Listeners should have the following function operator().
     *
     * PortActionListener::operator()(RTC::PortProfile pprof)
     *
     * The ownership of the given listener object is transferred to
     * this RTObject object in default.  The given listener object will
     * be destroied automatically in the RTObject's dtor or if the
     * listener is deleted by removePortActionListener() function.
     * If you want to keep ownership of the listener object, give
     * "false" value to 3rd argument to inhibit automatic destruction.
     *
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     * @param autoclean A flag for automatic listener destruction
     *
     * @endif
     */
    void 
    addPortActionListener(PortActionListenerType listener_type,
                          PortActionListener* listener,
                          bool autoclean = true);
    
    template <class Listener>
    PortActionListener*
    addPortActionListener(PortActionListenerType listener_type,
                          Listener& obj,
                          void (Listener::*memfunc)(const RTC::PortProfile&))
    {
      class Noname
        : public PortActionListener
      {
      public:
        Noname(Listener& obj,
               void (Listener::*memfunc)(const RTC::PortProfile&))
          : m_obj(obj), m_memfunc(memfunc)
        {
        }
        void operator()(const RTC::PortProfile& pprofile)
        {
          (m_obj.*m_memfunc)(pprofile);
        }
      private:
        Listener& m_obj;
        typedef void (Listener::*Memfunc)(const RTC::PortProfile&);
        Memfunc m_memfunc;
      };
      Noname* listener(new Noname(obj, memfunc));
      addPortActionListener(listener_type, listener, true);
      return listener;
    }

    /*!
     * @if jp
     * @brief PortActionListener Xi폜
     *
     * ݒ肵e탊Xi폜B
     * 
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     *
     * @else
     * @brief Removing PortAction type listener
     *
     * This operation removes a specified listener.
     *     
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     *
     * @endif
     */
    void 
    removePortActionListener(PortActionListenerType listener_type,
                             PortActionListener* listener);



    /*!
     * @if jp
     * @brief ExecutionContextActionListener Xiǉ
     *
     * ExecutionContext̒ǉA폜ɃR[obNe탊Xiݒ肷B
     *
     * ݒł郊Xĩ^CvƃR[obNCxg͈ȉ̒ʂ
     *
     * - ATTACH_EC:    ExecutionContext A^b`
     * - DETACH_EC:    ExecutionContext f^b`
     *
     * Xi ExecutionContextActionListener pAȉ̃VOj`
     * operator() ĂKvB
     *
     * ExecutionContextActionListener::operator()(UniqueId@ec_id)
     *
     * ftHgł́Å֐ɗ^XiIuWFNg̏L
     * RTObjectɈڂARTObject̎́A
     * removeExecutionContextActionListener() ɂ폜ɎIɉ̂B
     * XiIuWFNg̏LĂяoňێꍇ́A3
     *  false w肵AIȉ̂}邱ƂłB
     *
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     * @param autoclean XiIuWFNg̎Îsǂ̃tO
     *
     * @else
     * @brief Adding ExecutionContextAction type listener
     *
     * This operation adds certain listeners related to ComponentActions
     * post events.
     * The following listener types are available.
     *
     * - ADD_PORT:    At adding ExecutionContext
     * - REMOVE_PORT: At removing ExecutionContext
     *
     * Listeners should have the following function operator().
     *
     * ExecutionContextActionListener::operator()(UniqueId ec_id)
     *
     * The ownership of the given listener object is transferred to
     * this RTObject object in default.  The given listener object will
     * be destroied automatically in the RTObject's dtor or if the
     * listener is deleted by removeExecutionContextActionListener() function.
     * If you want to keep ownership of the listener object, give
     * "false" value to 3rd argument to inhibit automatic destruction.
     *
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     * @param autoclean A flag for automatic listener destruction
     *
     * @endif
     */
    typedef ExecutionContextActionListenerType ECActionListenerType;
    typedef ExecutionContextActionListener ECActionListener;
    void addExecutionContextActionListener(ECActionListenerType listener_type,
                                           ECActionListener* listener,
                                           bool autoclean = true);

    template <class Listener>
    ECActionListener*
    addExecutionContextActionListener(ECActionListenerType listener_type,
                                      Listener& obj,
                                      void (Listener::*memfunc)(UniqueId))
    {
      class Noname
        : public ECActionListener
      {
      public:
        Noname(Listener& obj, void (Listener::*memfunc)(UniqueId))
          : m_obj(obj), m_memfunc(memfunc)
        {
        }
        void operator()(UniqueId ec_id)
        {
          (m_obj.*m_memfunc)(ec_id);
        }
      private:
        Listener& m_obj;
        typedef void (Listener::*Memfunc)(UniqueId);
        Memfunc m_memfunc;
      };
      Noname* listener(new Noname(obj, memfunc));
      addExecutionContextActionListener(listener_type, listener, true);
      return listener;
    }
    

    /*!
     * @if jp
     * @brief ExecutionContextActionListener Xi폜
     *
     * ݒ肵e탊Xi폜B
     * 
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     *
     * @else
     * @brief Removing ExecutionContextAction type listener
     *
     * This operation removes a specified listener.
     *     
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     *
     * @endif
     */
    void 
    removeExecutionContextActionListener(ECActionListenerType listener_type,
                                         ECActionListener* listener);


    /*!
     * @if jp
     * @brief PortConnectListener Xiǉ
     *
     * Port̐ڑڑɌĂяoe탊Xiݒ肷B
     *
     * ݒł郊Xĩ^CvƃR[obNCxg͈ȉ̒ʂ
     *
     * - ON_NOTIFY_CONNECT: notify_connect() ֐Ăяo
     * - ON_NOTIFY_DISCONNECT: notify_disconnect() Ăяo
     * - ON_UNSUBSCRIBE_INTERFACES: notify_disconnect() IFwǉ
     *
     * Xi PortConnectListener pAȉ̃VOj`
     * operator() ĂKvB
     *
     * PortConnectListener::operator()(const char*, ConnectorProfile)
     *
     * ftHgł́Å֐ɗ^XiIuWFNg̏L
     * RTObjectɈڂARTObject̎́A
     * removePortConnectListener() ɂ폜ɎIɉ̂B
     * XiIuWFNg̏LĂяoňێꍇ́A3
     *  false w肵AIȉ̂}邱ƂłB
     *
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     * @param autoclean XiIuWFNg̎Îsǂ̃tO
     *
     * @else
     * @brief Adding PortConnect type listener
     *
     * This operation adds certain listeners related to Port's connect actions.
     * The following listener types are available.
     *
     * - ON_NOTIFY_CONNECT: right after entering into notify_connect()
     * - ON_NOTIFY_DISCONNECT: right after entering into notify_disconnect()
     * - ON_UNSUBSCRIBE_INTERFACES: unsubscribing IF in notify_disconnect()
     *
     * Listeners should have the following function operator().
     *
     * PortConnectListener::operator()(const char*, ConnectorProfile)
     *
     * The ownership of the given listener object is transferred to
     * this RTObject object in default.  The given listener object will
     * be destroied automatically in the RTObject's dtor or if the
     * listener is deleted by removePortConnectListener() function.
     * If you want to keep ownership of the listener object, give
     * "false" value to 3rd argument to inhibit automatic destruction.
     *
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     * @param autoclean A flag for automatic listener destruction
     *
     * @endif
     */
    void addPortConnectListener(PortConnectListenerType listener_type,
                                           PortConnectListener* listener,
                                           bool autoclean = true);

    template <class Listener>
    PortConnectListener*
    addPortConnectListener(PortConnectListenerType listener_type,
                           Listener& obj,
                           void (Listener::*memfunc)(const char*,
                                                     ConnectorProfile&))
    {
      class Noname
        : public PortConnectListener
      {
      public:
        Noname(Listener& obj,
               void (Listener::*memfunc)(const char*, ConnectorProfile&))
          : m_obj(obj), m_memfunc(memfunc)
        {
        }
        void operator()(const char* portname, ConnectorProfile& cprofile)
        {
          (m_obj.*m_memfunc)(portname, cprofile);
        }
      private:
        Listener& m_obj;
        typedef void (Listener::*Memfunc)(const char*, ConnectorProfile&);
        Memfunc m_memfunc;
      };
      Noname* listener(new Noname(obj, memfunc));
      addPortConnectListener(listener_type, listener, true);
      return listener;
    }
    

    /*!
     * @if jp
     * @brief PortConnectListener Xi폜
     *
     * ݒ肵e탊Xi폜B
     * 
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     *
     * @else
     * @brief Removing PortConnect type listener
     *
     * This operation removes a specified listener.
     *     
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     *
     * @endif
     */
    void 
    removePortConnectListener(PortConnectListenerType listener_type,
                              PortConnectListener* listener);

    /*!
     * @if jp
     * @brief PortConnectRetListener Xiǉ
     *
     * Port̐ڑڑɌĂяoe탊Xiݒ肷B
     *
     * ݒł郊Xĩ^CvƃR[obNCxg͈ȉ̒ʂ
     *
     * - ON_CONNECT_NEXTPORT: notify_connect() ̃JXP[hĂяo
     * - ON_SUBSCRIBE_INTERFACES: notify_connect() ̃C^[tF[Xwǒ
     * - ON_CONNECTED: nofity_connect() ڑɌĂяo
     * - ON_DISCONNECT_NEXT: notify_disconnect() ɃJXP[hĂяo
     * - ON_DISCONNECTED: notify_disconnect() ^[
     *
     * Xi PortConnectRetListener pAȉ̃VOj`
     * operator() ĂKvB
     *
     * PortConnectRetListener::operator()(const char*, ConnectorProfile)
     *
     * ftHgł́Å֐ɗ^XiIuWFNg̏L
     * RTObjectɈڂARTObject̎́A
     * removePortConnectRetListener() ɂ폜ɎIɉ̂B
     * XiIuWFNg̏LĂяoňێꍇ́A3
     *  false w肵AIȉ̂}邱ƂłB
     *
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     * @param autoclean XiIuWFNg̎Îsǂ̃tO
     *
     * @else
     * @brief Adding PortConnectRet type listener
     *
     * This operation adds certain listeners related to Port's connect actions.
     * The following listener types are available.
     *
     * - ON_CONNECT_NEXTPORT: after cascade-call in notify_connect()
     * - ON_SUBSCRIBE_INTERFACES: after IF subscribing in notify_connect()
     * - ON_CONNECTED: completed nofity_connect() connection process
     * - ON_DISCONNECT_NEXT: after cascade-call in notify_disconnect()
     * - ON_DISCONNECTED: completed notify_disconnect() disconnection process
     *
     * Listeners should have the following function operator().
     *
     * PortConnectRetListener::operator()(const char*, ConnectorProfile)
     *
     * The ownership of the given listener object is transferred to
     * this RTObject object in default.  The given listener object will
     * be destroied automatically in the RTObject's dtor or if the
     * listener is deleted by removePortConnectRetListener() function.
     * If you want to keep ownership of the listener object, give
     * "false" value to 3rd argument to inhibit automatic destruction.
     *
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     * @param autoclean A flag for automatic listener destruction
     *
     * @endif
     */
    void addPortConnectRetListener(PortConnectRetListenerType listener_type,
                                           PortConnectRetListener* listener,
                                           bool autoclean = true);

    template <class Listener>
    PortConnectRetListener*
    addPortConnectRetListener(PortConnectRetListenerType listener_type,
                              Listener& obj,
                              void (Listener::*memfunc)(const char*,
                                                        ConnectorProfile&,
                                                        ReturnCode_t))
    {
      class Noname
        : public PortConnectRetListener
      {
      public:
        Noname(Listener& obj,
               void (Listener::*memfunc)(const char*,
                                         ConnectorProfile&,
                                         ReturnCode_t))
          : m_obj(obj), m_memfunc(memfunc)
        {
        }
        void operator()(const char* portname,
                        ConnectorProfile& cprofile,
                        ReturnCode_t ret)
        {
          (m_obj.*m_memfunc)(portname, cprofile, ret);
        }
      private:
        Listener& m_obj;
        typedef void (Listener::*Memfunc)(const char* portname,
                                          ConnectorProfile& cprofile,
                                          ReturnCode_t ret);
        Memfunc m_memfunc;
      };
      Noname* listener(new Noname(obj, memfunc));
      addPortConnectRetListener(listener_type, listener, true);
      return listener;
    }
    

    /*!
     * @if jp
     * @brief PortConnectRetListener Xi폜
     *
     * ݒ肵e탊Xi폜B
     * 
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     *
     * @else
     * @brief Removing PortConnectRet type listener
     *
     * This operation removes a specified listener.
     *     
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     *
     * @endif
     */
    void 
    removePortConnectRetListener(PortConnectRetListenerType listener_type,
                                 PortConnectRetListener* listener);


    /*!
     * @if jp
     *
     * @brief ConfigurationParamListener ǉ
     *
     * update(const char* config_set, const char* config_param) Ă΂ꂽۂ
     * R[郊Xi ConfigurationParamListener ǉB
     * type ɂ݂͌̂Ƃ ON_UPDATE_CONFIG_PARAM ݂̂B
     *
     * @param type ConfigurationParamListenerType^̒lB
     *             ON_UPDATE_CONFIG_PARAM B
     *
     * @param listener ConfigurationParamListener ^̃XiIuWFNgB
     * @param autoclean XiIuWFNgō폜邩ǂ̃tO
     * 
     * @else
     *
     * @brief Adding ConfigurationParamListener 
     * 
     * This function adds a listener object which is called when
     * update(const char* config_set, const char* config_param) is
     * called. In the type argument, currently only
     * ON_UPDATE_CONFIG_PARAM is allowed.
     *
     * @param type ConfigurationParamListenerType value
     *             ON_UPDATE_CONFIG_PARAM is only allowed.
     *
     * @param listener ConfigurationParamListener listener object.
     * @param autoclean a flag whether if the listener object autocleaned.
     *
     * @endif
     */
    void addConfigurationParamListener(ConfigurationParamListenerType type,
                                       ConfigurationParamListener* listener,
                                       bool autoclean = true);

    template <class Listener>
    ConfigurationParamListener*
    addConfigurationParamListener(ConfigurationParamListenerType listener_type,
                                  Listener& obj,
                                  void (Listener::*memfunc)(const char*,
                                                            const char*))
    {
      class Noname
        : public ConfigurationParamListener
      {
      public:
        Noname(Listener& obj,
               void (Listener::*memfunc)(const char*, const char*))
          : m_obj(obj), m_memfunc(memfunc)
        {
        }
        void operator()(const char* config_set_name,
                        const char* config_param_name)
        {
          (m_obj.*m_memfunc)(config_set_name, config_param_name);
        }
      private:
        Listener& m_obj;
        typedef void (Listener::*Memfunc)(const char*, const char*);
        Memfunc m_memfunc;
      };
      Noname* listener(new Noname(obj, memfunc));
      addConfigurationParamListener(listener_type, listener, true);
      return listener;
    }

    /*!
     * @if jp
     *
     * @brief ConfigurationParamListener 폜
     *
     * addConfigurationParamListener ŒǉꂽXiIuWFNg폜B
     *
     * @param type ConfigurationParamListenerType^̒lB
     *             ON_UPDATE_CONFIG_PARAM B
     * @param listener ^XiIuWFNgւ̃|C^
     * 
     * @else
     *
     * @brief Removing ConfigurationParamListener 
     * 
     * This function removes a listener object which is added by
     * addConfigurationParamListener() function.
     *
     * @param type ConfigurationParamListenerType value
     *             ON_UPDATE_CONFIG_PARAM is only allowed.
     * @param listener a pointer to ConfigurationParamListener listener object.
     *
     * @endif
     */
    void removeConfigurationParamListener(ConfigurationParamListenerType type,
                                          ConfigurationParamListener* listener);
    
    /*!
     * @if jp
     *
     * @brief ConfigurationSetListener ǉ
     *
     * ConfigurationSet XVꂽƂȂǂɌĂ΂郊Xi
     * ConfigurationSetListener ǉBݒ\ȃCxg͈ȉ
     * 2ނB
     *
     * - ON_SET_CONFIG_SET: setConfigurationSetValues() 
     *                      ConfigurationSet ɒlݒ肳ꂽꍇB
     * - ON_ADD_CONFIG_SET: addConfigurationSet() ŐV
     *                      ConfigurationSet ǉꂽꍇB
     *
     * @param type ConfigurationSetListenerType^̒lB
     * @param listener ConfigurationSetListener ^̃XiIuWFNgB
     * @param autoclean XiIuWFNgō폜邩ǂ̃tO
     * 
     * @else
     *
     * @brief Adding ConfigurationSetListener 
     * 
     * This function add a listener object which is called when
     * ConfigurationSet is updated. Available events are the followings.
     *
     * @param type ConfigurationSetListenerType value
     * @param listener ConfigurationSetListener listener object.
     * @param autoclean a flag whether if the listener object autocleaned.
     *
     * @endif
     */
    void addConfigurationSetListener(ConfigurationSetListenerType type,
                                     ConfigurationSetListener* listener,
                                     bool autoclean = true);

    template <class Listener>
    ConfigurationSetListener*
    addConfigurationSetListener(ConfigurationSetListenerType listener_type,
                                Listener& obj,
                                void (Listener::*memfunc)
                                (const coil::Properties& config_set))
    {
      class Noname
        : public ConfigurationSetListener
      {
      public:
        Noname(Listener& obj,
               void (Listener::*memfunc)(const coil::Properties& config_set))
          : m_obj(obj), m_memfunc(memfunc)
        {
        }
        virtual void operator()(const coil::Properties& config_set)
        {
          (m_obj.*m_memfunc)(config_set);
        }
      private:
        Listener& m_obj;
        typedef void (Listener::*Memfunc)(const coil::Properties& config_set);
        Memfunc m_memfunc;
      };
      Noname* listener(new Noname(obj, memfunc));
      addConfigurationSetListener(listener_type, listener, true);
      return listener;
    }

    /*!
     * @if jp
     *
     * @brief ConfigurationSetListener 폜
     *
     * addConfigurationSetListener ŒǉꂽXiIuWFNg폜B
     *
     * @param type ConfigurationSetListenerType^̒lB
     * @param listener ^XiIuWFNgւ̃|C^
     * 
     * @else
     *
     * @brief Removing ConfigurationSetListener 
     * 
     * This function removes a listener object which is added by
     * addConfigurationSetListener() function.
     *
     * @param type ConfigurationSetListenerType value
     * @param listener a pointer to ConfigurationSetListener listener object.
     *
     * @endif
     */
    void removeConfigurationSetListener(ConfigurationSetListenerType type,
                                        ConfigurationSetListener* listener);
    
    /*!
     * @if jp
     *
     * @brief ConfigurationSetNameListener ǉ
     *
     * ConfigurationSetName XVꂽƂȂǂɌĂ΂郊Xi
     * ConfigurationSetNameListener ǉBݒ\ȃCxg͈ȉ
     * 3ނB
     *
     * - ON_UPDATE_CONFIG_SET:  ConfigurationSet Abvf[gꂽ
     * - ON_REMOVE_CONFIG_SET:  ConfigurationSet 폜ꂽ
     * - ON_ACTIVATE_CONFIG_SET:  ConfigurationSet ANeBuꂽ
     *
     * @param type ConfigurationSetNameListenerType^̒lB
     * @param listener ConfigurationSetNameListener ^̃XiIuWFNgB
     * @param autoclean XiIuWFNgō폜邩ǂ̃tO
     * 
     * @else
     *
     * @brief Adding ConfigurationSetNameListener 
     * 
     * This function add a listener object which is called when
     * ConfigurationSetName is updated. Available events are the followings.
     *
     * - ON_UPDATE_CONFIG_SET: A ConfigurationSet has been updated.
     * - ON_REMOVE_CONFIG_SET: A ConfigurationSet has been deleted.
     * - ON_ACTIVATE_CONFIG_SET: A ConfigurationSet has been activated.
     *
     * @param type ConfigurationSetNameListenerType value
     * @param listener ConfigurationSetNameListener listener object.
     * @param autoclean a flag whether if the listener object autocleaned.
     *
     * @endif
     */
    void 
    addConfigurationSetNameListener(ConfigurationSetNameListenerType type,
                                    ConfigurationSetNameListener* listener,
                                    bool autoclean = true);

    template <class Listener>
    ConfigurationSetNameListener*
    addConfigurationSetNameListener(ConfigurationSetNameListenerType type,
                                    Listener& obj,
                                    void (Listener::*memfunc)(const char*))
    {
      class Noname
        : public ConfigurationSetNameListener
      {
      public:
        Noname(Listener& obj, void (Listener::*memfunc)(const char*))
          : m_obj(obj), m_memfunc(memfunc)
        {
        }
        virtual void operator()(const char* config_set_name)
        {
          (m_obj.*m_memfunc)(config_set_name);
        }
      private:
        Listener& m_obj;
        typedef void (Listener::*Memfunc)(const char*);
        Memfunc m_memfunc;
      };
      Noname* listener(new Noname(obj, memfunc));
      addConfigurationSetNameListener(type, listener, true);
      return listener;
    }

    /*!
     * @if jp
     *
     * @brief ConfigurationSetNameListener 폜
     *
     * addConfigurationSetNameListener ŒǉꂽXiIuWFNg
     * 폜B
     *
     * @param type ConfigurationSetNameListenerType^̒lB
     *             ON_UPDATE_CONFIG_PARAM B
     * @param listener ^XiIuWFNgւ̃|C^
     * 
     * @else
     *
     * @brief Removing ConfigurationSetNameListener 
     * 
     * This function removes a listener object which is added by
     * addConfigurationSetNameListener() function.
     *
     * @param type ConfigurationSetNameListenerType value
     *             ON_UPDATE_CONFIG_PARAM is only allowed.
     * @param listener a pointer to ConfigurationSetNameListener
     *             listener object.
     *
     * @endif
     */
    void
    removeConfigurationSetNameListener(ConfigurationSetNameListenerType type,
                                       ConfigurationSetNameListener* listener);
    
  protected:
    /*!
     * @if jp
     *
     * @brief RTC I
     *
     * RTC ̏IsB
     * ێĂS Port ̓o^ƂƂɁAY CORBA IuWFNg
     * 񊈐ARTC IB
     *
     * @else
     *
     * @brief Shutdown RTC
     *
     * This operation ececutes RTC's termination.
     * This unregisters all Ports, deactivates corresponding CORBA objects and 
     * shuts down RTC.
     *
     * @endif
     */
    void shutdown();

    inline void preOnInitialize(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_INITIALIZE].notify(ec_id);
    }

    inline void preOnFinalize(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_FINALIZE].notify(ec_id);
    }

    inline void preOnStartup(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_STARTUP].notify(ec_id);
    }

    inline void preOnShutdown(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_SHUTDOWN].notify(ec_id);
    }

    inline void preOnActivated(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_ACTIVATED].notify(ec_id);
    }

    inline void preOnDeactivated(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_DEACTIVATED].notify(ec_id);
    }

    inline void preOnAborting(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_ABORTING].notify(ec_id);
    }

    inline void preOnError(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_ERROR].notify(ec_id);
    }

    inline void preOnReset(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_RESET].notify(ec_id);
    }

    inline void preOnExecute(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_EXECUTE].notify(ec_id);
    }

    inline void preOnStateUpdate(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_STATE_UPDATE].notify(ec_id);
    }

    inline void preOnRateChanged(UniqueId ec_id)
    {
      m_actionListeners.preaction_[PRE_ON_RATE_CHANGED].notify(ec_id);
    }

    inline void postOnInitialize(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_INITIALIZE].notify(ec_id, ret);
    }

    inline void postOnFinalize(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_FINALIZE].notify(ec_id, ret);
    }

    inline void postOnStartup(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_STARTUP].notify(ec_id, ret);
    }

    inline void postOnShutdown(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_SHUTDOWN].notify(ec_id, ret);
    }

    inline void postOnActivated(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_ACTIVATED].notify(ec_id, ret);
    }

    inline void postOnDeactivated(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_DEACTIVATED].notify(ec_id, ret);
    }

    inline void postOnAborting(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_ABORTING].notify(ec_id, ret);
    }

    inline void postOnError(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_ERROR].notify(ec_id, ret);
    }

    inline void postOnReset(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_RESET].notify(ec_id, ret);
    }

    inline void postOnExecute(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_EXECUTE].notify(ec_id, ret);
    }

    inline void postOnStateUpdate(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_STATE_UPDATE].notify(ec_id, ret);
    }

    inline void postOnRateChanged(UniqueId ec_id, ReturnCode_t ret)
    {
      m_actionListeners.postaction_[POST_ON_RATE_CHANGED].notify(ec_id, ret);
    }

    inline void onAddPort(const PortProfile& pprof)
    {
      m_actionListeners.portaction_[ADD_PORT].notify(pprof);
    }
    
    inline void onRemovePort(const PortProfile& pprof)
    {
      m_actionListeners.portaction_[REMOVE_PORT].notify(pprof);
    }
    
    inline void onAttachExecutionContext(UniqueId ec_id)
    {
      m_actionListeners.ecaction_[EC_ATTACHED].notify(ec_id);
    }
    
    inline void onDetachExecutionContext(UniqueId ec_id)
    {
      m_actionListeners.ecaction_[EC_DETACHED].notify(ec_id);
    }
    
  protected:
    /*!
     * @if jp
     * @brief K[Xg[
     * @else
     * @brief Logger stream
     * @endif
     */
    mutable Logger rtclog;
    /*!
     * @if jp
     * @brief }l[WIuWFNg
     * @else
     * @brief Manager object
     * @endif
     */
    Manager* m_pManager;
    
    /*!
     * @if jp
     * @brief ORB ւ̃|C^
     * @else
     * @brief The pointer to the ORB
     * @endif
     */
    CORBA::ORB_var m_pORB;
    
    /*!
     * @if jp
     * @brief POA ւ̃|C^
     * @else
     * @brief The pointer to the POA
     * @endif
     */
    PortableServer::POA_var m_pPOA;
    
    //============================================================
    // SDO ֌W̕ϐ
    //============================================================
    /*!
     * @if jp
     * @brief SDO ێ organization ̃Xg
     * @else
     * @brief SDO owned organization list
     * @endif
     */
    SDOPackage::OrganizationList m_sdoOwnedOrganizations;
    
    /*!
     * @if jp
     * @brief SDOService ̃vt@CXgidŃT[`邽߂̃t@N^
     * @else
     * @brief Functor to find from SDOService Profile List by id
     * @endif
     */
    struct svc_name
    {
      svc_name (const char* id) : m_id(id) {};
      bool operator()(const SDOPackage::ServiceProfile& prof)
      {
	return m_id == std::string(prof.id);
      }
      std::string m_id;
    };  // struct svc_name
    
    /*!
     * @if jp
     * @brief SDO Configuration IuWFNgւ̃|C^
     * @else
     * @brief The pointer to the SDO Configuration Object
     * @endif
     */
    SDOPackage::Configuration_impl* m_pSdoConfigImpl;
    
    /*!
     * @if jp
     * @brief SDO Configuration Interface ւ̃|C^
     * @else
     * @brief The pointer to the SDO Configuration Interface
     * @endif
     */
    SDOPackage::Configuration_var  m_pSdoConfig;
    
    /*!
     * @if jp
     * @brief SDO organization
     * @else
     * @brief SDO organization
     * @endif
     */
    SDOPackage::OrganizationList m_sdoOrganizations;
    
    /*!
     * @if jp
     * @brief SDO Status
     * @else
     * @brief SDO Status
     * @endif
     */
    SDOPackage::NVList m_sdoStatus;
    
    //============================================================
    // RTC ֌W̕ϐ
    //============================================================
    /*!
     * @if jp
     * @brief R|[lgvt@C
     * @else
     * @brief ComponentProfile
     * @endif
     */
    ComponentProfile m_profile;
    
    /*!
     * @if jp
     * @brief IuWFNgt@X
     * @else
     * @brief Object reference
     * @endif
     */
    RTObject_var m_objref;
    
    /*!
     * @if jp
     * @brief Port ̃IuWFNgt@X̃Xg
     * @else
     * @brief List of Port Object reference
     * @endif
     */
    PortAdmin m_portAdmin;

    /*!
     * @if jp
     * @brief InPortBase* ̃Xg
     * @else
     * @brief List of InPortBase*
     * @endif
     */
    std::vector<InPortBase*> m_inports;

    /*!
     * @if jp
     * @brief OutPortBase* ̃Xg
     * @else
     * @brief List of OutPortBase*
     * @endif
     */
    std::vector<OutPortBase*> m_outports;
    
    /*!
     * @if jp
     * @brief ownerExecutionContextService ̃Xg
     * @else
     * @brief List of owned ExecutionContextService
     * @endif
     */
    ExecutionContextServiceList m_ecMine;
    
    /*!
     * @if jp
     * @brief ExecutionContextBase ̃Xg
     * @else
     * @brief List of ExecutionContextBase 
     * @endif
     */
    std::vector<ExecutionContextBase*> m_eclist;
    
    /*!
     * @if jp
     * @brief QĂExecutionContextService ̃Xg
     * @else
     * @brief List of participating ExecutionContextService
     * @endif
     */
    ExecutionContextServiceList m_ecOther;
    
    /*!
     * @if jp
     * @brief Created ԃtO
     * @else
     * @brief Created Status Flag
     * @endif
     */
    bool m_created;
    
    /*!
     * @if jp
     * @brief RTC̏IԃtO
     * @else
     * @brief RTC Finalize Status Flag
     * @endif
     */
    bool m_exiting;
    
    /*!
     * @if jp
     * @brief Alive ԃtO
     * @else
     * @brief Alive Status Flag
     * @endif
     */
    //    bool m_alive;
    
    /*!
     * @if jp
     * @brief RTC ̃vpeB
     * @else
     * @brief RTC's Property
     * @endif
     */
    coil::Properties m_properties;
    
    /*!
     * @if jp
     * @brief RtBM[VǗIuWFNg
     * @else
     * @brief Configuration Administrator Object
     * @endif
     */
    ConfigAdmin m_configsets;
    
    /*!
     * @if jp
     * @brief SDO Service ǗIuWFNg
     * @else
     * @brief SDO Service Administrator Object
     * @endif
     */
    SdoServiceAdmin m_sdoservice;

    /*!
     * @if jp
     * @brief readAll()ďop̃tO
     * @else
     * @brief flag for readAll()
     * @endif
     */
    bool m_readAll;

    /*!
     * @if jp
     * @brief writeAll()ďop̃tO
     * @else
     * @brief flag for writeAll()
     * @endif
     */
    bool m_writeAll;

    /*!
     * @if jp
     * @brief readAll()p̃tO
     *
     * true:readAll()̓rłŃG[ĂŌ܂Ŏ{B
     * false:readAll()̓rłŃG[ꍇIB
     *
     * @else
     * @brief flag for readAll()
     *
     * true:Even if the error occurs during readAll(), it executes it to the 
     *      last minute. 
     * false:End when error occurs during readAll().
     *
     * @endif
     */
    bool m_readAllCompletion;

    /*!
     * @if jp
     * @brief writeAll()p̃tO
     *
     * true:writeAll()̓rłŃG[ĂŌ܂Ŏ{B
     * false:writeAll()̓rłŃG[ꍇIB
     *
     * @else
     * @brief flag for writeAll()
     *
     * true:Even if the error occurs during writeAll(), it executes it to the 
     *      last minute. 
     * false:End when error occurs during writeAll().
     *
     * @endif
     */
    bool m_writeAllCompletion;

    /*!
     * @if jp
     * @brief ComponentActionListenerz_
     *
     * ComponentActionListenrێz_
     *
     * @else
     * @brief ComponentActionListener holder
     *
     * Holders of ComponentActionListeners
     *
     * @endif
     */
    ComponentActionListeners m_actionListeners;

    /*!
     * @if jp
     * @brief PortConnectListenerz_
     *
     * PortConnectListenrێz_
     *
     * @else
     * @brief PortConnectListener holder
     *
     * Holders of PortConnectListeners
     *
     * @endif
     */
    PortConnectListeners m_portconnListeners;

    //------------------------------------------------------------
    // Functor
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief NVList pt@N^
     * @else
     * @brief Functor to find NVList
     * @endif
     */
    struct nv_name
    {
      nv_name(const char* name) : m_name(name) {};
      bool operator()(const SDOPackage::NameValue& nv)
      {
	return m_name == std::string(nv.name);
      }
      std::string m_name;
    };  // struct nv_name
    
    /*!
     * @if jp
     * @brief ExecutionContext Rs[t@N^
     * @else
     * @brief Functor to copy ExecutionContext
     * @endif
     */
    struct ec_copy
    {
      ec_copy(ExecutionContextList& eclist)
	: m_eclist(eclist)
      {
      }
      void operator()(ExecutionContextService_ptr ecs)
      {
        if (!::CORBA::is_nil(ecs))
          {
	    CORBA_SeqUtil::push_back(m_eclist,
                                     ExecutionContext::_duplicate(ecs));
          }
      }
      ExecutionContextList& m_eclist;
    };  // struct ec_copy
    /*!
     * @if jp
     * @brief ExecutionContext pt@N^
     * @else
     * @brief Functor to find ExecutionContext
     * @endif
     */
    struct ec_find
    {
      ec_find(ExecutionContext_ptr& ec)
	: m_ec(ExecutionContext::_duplicate(ec))
      {
      }
      bool operator()(ExecutionContextService_ptr ecs)
      {
	try
	  {
            if (!::CORBA::is_nil(ecs))
              {
  	        ExecutionContext_var ec;
	        ec = ExecutionContext::_narrow(ecs);
	        return m_ec->_is_equivalent(ec);
              }
	  }
	catch (...)
	  {
	    return false;
	  }
	return false;
      }
      ExecutionContext_var m_ec;

    };  // struct ec_find
    //    ExecutionContextAdminList m_execContextList;
    
    /*!
     * @if jp
     * @brief RTC 񊈐pt@N^
     * @else
     * @brief Functor to deactivate RTC
     * @endif
     */
    struct deactivate_comps
    {
      deactivate_comps(LightweightRTObject_ptr comp)
	: m_comp(RTC::LightweightRTObject::_duplicate(comp))
      {
      }
      void operator()(ExecutionContextService_ptr ec)
      {
        if (!::CORBA::is_nil(ec) && !ec->_non_existent())
          {
            
	    ec->deactivate_component(RTC::LightweightRTObject::_duplicate(m_comp));
            ec->stop();
          }
      }
      LightweightRTObject_var m_comp;
    };  // struct deactivate_comps
  };  // class RTObject_impl
};  // namespace RTC

#ifdef WIN32
#pragma warning( default : 4290 )
#endif

#endif // RTC_RTOBJECT
